# -*- coding: utf-8 -*-
import collections
import copy
import itertools
import types
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import systemtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import select


class Selection(object):
    r'''A selection of components.

    ::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> selection = selectiontools.Selection(staff[:])
        >>> selection
        Selection([Note("c'4"), Note("d'4"), Note("e'4"), Note("f'4")])

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_music',
        )

    ### INITIALIZER ###

    def __init__(self, music=None):
        music = self._coerce_music(music)
        self._music = tuple(music)

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Cocatenates `argument` to selection.

        Returns new selection.
        '''
        assert isinstance(argument, (type(self), list, tuple))
        if isinstance(argument, type(self)):
            music = self._music + argument._music
            return type(self)(music)
        elif isinstance(argument, (tuple, list)):
            music = self._music + tuple(argument)
        return type(self)(music)

    def __contains__(self, argument):
        r'''Is true when `argument` is in selection. Otherwise false.

        Returns true or false.
        '''
        return argument in self._music

    def __eq__(self, argument):
        r'''Is true when selection and `argument` are of the same type
        and when music of selection equals music of `argument`.
        Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self._music == argument._music
        elif isinstance(argument, collections.Sequence):
            return self._music == tuple(argument)
        return False

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        Returns component from selection.
        '''
        result = self._music.__getitem__(argument)
        if isinstance(result, tuple):
            selection = type(self)()
            selection._music = result[:]
            result = selection
        return result

    def __getstate__(self):
        r'''Gets state of selection.

        Returns dictionary.
        '''
        if hasattr(self, '__dict__'):
            state = vars(self).copy()
        else:
            state = {}
        for class_ in type(self).__mro__:
            for slot in getattr(class_, '__slots__', ()):
                try:
                    state[slot] = getattr(self, slot)
                except AttributeError:
                    pass
        return state

    def __hash__(self):
        r'''Hashes selection.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        hash_values = (type(self), self._music)
        return hash(hash_values)

    def __illustrate__(self):
        r'''Attempts to illustrate selection.

        Evaluates the storage format of the selection (to sever any references
        to the source score from which the selection was taken). Then tries to
        wrap the result in a staff; in the case that notes of only C4 are found
        then sets the staff context name to ``'RhythmicStaff'``. If this works
        then the staff is wrapped in a LilyPond file and the file is returned.
        If this doesn't work then the method raises an exception.

        The idea is that the illustration should work for simple selections of
        that represent an essentially contiguous snippet of a single voice of
        music.

        Returns LilyPond file.
        '''
        import abjad
        music = abjad.mutate(self).copy()
        staff = abjad.Staff(music)
        found_different_pitch = False
        for pitch in abjad.iterate(staff).by_pitch():
            if pitch != abjad.NamedPitch("c'"):
                found_different_pitch = True
                break
        if not found_different_pitch:
            staff.context_name = 'RhythmicStaff'
        score = abjad.Score([staff])
        lilypond_file = abjad.LilyPondFile.new(score)
        lilypond_file.header_block.tagline = False
        return lilypond_file

    def __len__(self):
        r'''Gets number of components in selection.

        Returns nonnegative integer.
        '''
        return len(self._music)

    def __ne__(self, argument):
        r'''Is true when selection does not equal `argument`. Otherwise false.

        Returns true or false.
        '''
        return not self == argument

    def __radd__(self, argument):
        r'''Concatenates selection to `argument`.

        Returns newly created selection.
        '''
        assert isinstance(argument, (type(self), list, tuple))
        if isinstance(argument, type(self)):
            music = argument._music + self._music
            return type(self)(music)
        elif isinstance(argument, (tuple, list)):
            music = tuple(argument) + self._music
        return type(self)(music)

    def __repr__(self):
        r'''Gets interpreter representation of selection.

        Returns string.
        '''
        return systemtools.StorageFormatAgent(self).get_repr_format()

    def __setstate__(self, state):
        r'''Sets state of selection.

        Returns none.
        '''
        for key, value in state.items():
            setattr(self, key, value)

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_components_in_same_logical_voice(
        argument, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(argument, allowable_types):
            return False
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(argument) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in argument:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        first = argument[0]
        if not isinstance(first, prototype):
            return False
        orphan_components = True
        if not first._get_parentage().is_orphan:
            orphan_components = False
        same_logical_voice = True
        first_signature = first._get_parentage().logical_voice
        for component in argument[1:]:
            parentage = component._get_parentage()
            if not parentage.is_orphan:
                orphan_components = False
            if not allow_orphans and orphan_components:
                return False
            if parentage.logical_voice != first_signature:
                same_logical_voice = False
            if not allow_orphans and not same_logical_voice:
                return False
            if allow_orphans and not orphan_components and \
                not same_logical_voice:
                return False
        return True

    @staticmethod
    def _all_are_contiguous_components_in_same_logical_voice(
        argument, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(argument, allowable_types):
            return False
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(argument) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in argument:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        if not allow_orphans:
            if any(x._get_parentage().is_orphan for x in argument):
                return False
        first = argument[0]
        if not isinstance(first, prototype):
            return False
        first_parentage = first._get_parentage()
        first_logical_voice = first_parentage.logical_voice
        first_root = first_parentage.root
        previous = first
        for current in argument[1:]:
            current_parentage = current._get_parentage()
            current_logical_voice = \
                current_parentage.logical_voice
            # false if wrong type of component found
            if not isinstance(current, prototype):
                return False
            # false if in different logical voices
            if current_logical_voice != first_logical_voice:
                return False
            # false if components are in same score and are discontiguous
            if current_parentage.root == first_root:
                if not previous._is_immediate_temporal_successor_of(current):
                    return False
            previous = current
        return True

    @staticmethod
    def _all_are_contiguous_components_in_same_parent(
        argument, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(argument, allowable_types):
            return False
        prototype = prototype or (scoretools.Component, )
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(argument) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in argument:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        first = argument[0]
        if not isinstance(first, prototype):
            return False
        first_parent = first._parent
        if first_parent is None:
            if allow_orphans:
                orphan_components = True
            else:
                return False
        same_parent = True
        strictly_contiguous = True
        previous = first
        for current in argument[1:]:
            if not isinstance(current, prototype):
                return False
            if not current._get_parentage().is_orphan:
                orphan_components = False
            if current._parent is not first_parent:
                same_parent = False
            if not previous._is_immediate_temporal_successor_of(current):
                strictly_contiguous = False
            if (not allow_orphans or
                (allow_orphans and not orphan_components)) and \
                (not same_parent or not strictly_contiguous):
                return False
            previous = current
        return True

    def _attach_tie_spanner_to_leaf_pair(self, use_messiaen_style_ties=False):
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        assert len(self) == 2
        left_leaf, right_leaf = self
        assert isinstance(left_leaf, scoretools.Leaf), left_leaf
        assert isinstance(right_leaf, scoretools.Leaf), right_leaf
        left_logical_tie = left_leaf._get_logical_tie()
        right_logical_tie = right_leaf._get_logical_tie()
        prototype = (spannertools.Tie,)
        if left_logical_tie == right_logical_tie:
            return
        try:
            left_tie_spanner = left_leaf._get_spanner(prototype)
        except MissingSpannerError:
            left_tie_spanner = None
        try:
            right_tie_spanner = right_leaf._get_spanner(prototype)
        except MissingSpannerError:
            right_tie_spanner = None
        if left_tie_spanner is not None and right_tie_spanner is not None:
            left_tie_spanner._fuse_by_reference(right_tie_spanner)
        elif left_tie_spanner is not None and right_tie_spanner is None:
            left_tie_spanner._append(right_leaf)
        elif left_tie_spanner is None and right_tie_spanner is not None:
            right_tie_spanner._append_left(left_leaf)
        elif left_tie_spanner is None and right_tie_spanner is None:
            tie = spannertools.Tie(
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            attach(tie, [left_leaf, right_leaf])

    @staticmethod
    def _coerce_music(music):
        if music is None:
            music = ()
        elif isinstance(music, Selection):
            music = tuple(music)
        elif isinstance(music, collections.Sequence):
            music = tuple(music)
        elif isinstance(music, types.GeneratorType):
            music = tuple(music)
        else:
            music = (music,)
        return music

    def _copy(self, n=1, include_enclosing_containers=False):
        r'''Copies components in selection and fractures crossing spanners.

        Components in selection must be logical-voice-contiguous.

        The steps this function takes are as follows:

            * Deep copy `components`.

            * Deep copy spanners that attach to any component in `components`.

            * Fracture spanners that attach to components not in `components`.

            * Returns Python list of copied components.

        ..  container:: example

            **Example 1.** Copy components one time:

            ::

                >>> staff = Staff(r"c'8 ( d'8 e'8 f'8 )")
                >>> staff.append(r"g'8 a'8 b'8 c''8")
                >>> time_signature = TimeSignature((2, 4))
                >>> attach(time_signature, staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    \time 2/4
                    c'8 (
                    d'8
                    e'8
                    f'8 )
                    g'8
                    a'8
                    b'8
                    c''8
                }

            ::

                >>> selection = staff[2:4]
                >>> result = selection._copy()
                >>> new_staff = Staff(result)
                >>> show(new_staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(new_staff))
                \new Staff {
                    e'8 (
                    f'8 )
                }

            ::

                >>> staff[2] is new_staff[0]
                False

        ..  container:: example

            **Example 2.** Copy components multiple times:

            Copy `components` a total of `n` times:

            ::

                >>> selection = staff[2:4]
                >>> result = selection._copy(n=4)
                >>> new_staff = Staff(result)
                >>> show(new_staff) # doctest: +SKIP

            ::

                >>> print(format(new_staff))
                \new Staff {
                    e'8 (
                    f'8 )
                    e'8 (
                    f'8 )
                    e'8 (
                    f'8 )
                    e'8 (
                    f'8 )
                }

        ..  container:: example

            **Example 3.** Copy leaves and include enclosing conatiners:

                >>> voice = Voice(r"\times 2/3 { c'4 d'4 e'4 }")
                >>> voice.append(r"\times 2/3 { f'4 e'4 d'4 }")
                >>> staff = Staff([voice])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    \new Voice {
                        \times 2/3 {
                            c'4
                            d'4
                            e'4
                        }
                        \times 2/3 {
                            f'4
                            e'4
                            d'4
                        }
                    }
                }

            ::

                >>> selector = select().by_leaf(flatten=True)
                >>> leaves = selector(staff)
                >>> leaves = leaves[1:5]
                >>> new_staff = leaves._copy(include_enclosing_containers=True)
                >>> show(new_staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(new_staff))
                \new Staff {
                    \new Voice {
                        \tweak edge-height #'(0.7 . 0)
                        \times 2/3 {
                            d'4
                            e'4
                        }
                        \tweak edge-height #'(0.7 . 0)
                        \times 2/3 {
                            f'4
                            e'4
                        }
                    }
                }

        Returns contiguous selection.
        '''
        # check input
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        # return empty list when nothing to copy
        if n < 1:
            return []
        new_components = [
            component._copy_with_children_and_indicators_but_without_spanners()
            for component in self
            ]
        if include_enclosing_containers:
            return self._copy_and_include_enclosing_containers()
        new_components = type(self)(new_components)
        # make schema of spanners contained by components
        schema = self._make_spanner_schema()
        # copy spanners covered by components
        for covered_spanner, component_indices in list(schema.items()):
            new_covered_spanner = copy.copy(covered_spanner)
            del(schema[covered_spanner])
            schema[new_covered_spanner] = component_indices
        # reverse schema
        reversed_schema = {}
        for new_covered_spanner, component_indices in list(schema.items()):
            for component_index in component_indices:
                try:
                    reversed_schema[component_index].append(
                        new_covered_spanner)
                except KeyError:
                    reversed_schema[component_index] = [new_covered_spanner]
        # iterate components and add new components to new spanners
        for component_index, new_component in enumerate(
            iterate(new_components).by_class()):
            try:
                new_covered_spanners = reversed_schema[component_index]
                for new_covered_spanner in new_covered_spanners:
                    new_covered_spanner._append(new_component)
            except KeyError:
                pass
        # repeat as specified by input
        for i in range(n - 1):
            new_components += self._copy()
        # return new components
        return new_components

    def _copy_and_include_enclosing_containers(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        # get governor
        parentage = self[0]._get_parentage(include_self=True)
        governor = parentage._get_governor()
        # find start and stop indices in governor
        governor_leaves = select(governor).by_leaf()
        for i, x in enumerate(governor_leaves):
            if x is self[0]:
                start_index_in_governor = i
        for i, x in enumerate(governor_leaves):
            if x is self[-1]:
                stop_index_in_governor = i
        # copy governor
        governor_copy = mutate(governor).copy()
        copied_leaves = select(governor_copy).by_leaf()
        # find start and stop leaves in copy of governor
        start_leaf = copied_leaves[start_index_in_governor]
        stop_leaf = copied_leaves[stop_index_in_governor]
        # trim governor copy forwards from first leaf
        found_start_leaf = False
        while not found_start_leaf:
            leaf = next(iterate(governor_copy).by_leaf())
            if leaf is start_leaf:
                found_start_leaf = True
            else:
                leaf._remove_and_shrink_durated_parent_containers()
        # trim governor copy backwards from last leaf
        found_stop_leaf = False
        while not found_stop_leaf:
            reverse_iterator = iterate(governor_copy).by_leaf(reverse=True)
            leaf = next(reverse_iterator)
            if leaf is stop_leaf:
                found_stop_leaf = True
            else:
                leaf._remove_and_shrink_durated_parent_containers()
        # return trimmed governor copy
        return governor_copy

    def _fuse(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        if all(isinstance(x, scoretools.Leaf) for x in self):
            return self._fuse_leaves()
        elif all(isinstance(x, scoretools.Tuplet) for x in self):
            return self._fuse_tuplets()
        elif all(isinstance(x, scoretools.Measure) for x in self):
            return self._fuse_measures()
        else:
            message = 'can not fuse.'
            raise Exception(message)

    def _fuse_leaves(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        assert all(isinstance(x, scoretools.Leaf) for x in self)
        leaves = self
        if len(leaves) <= 1:
            return leaves
        total_preprolated = leaves._preprolated_duration
        for leaf in leaves[1:]:
            parent = leaf._parent
            if parent:
                index = parent.index(leaf)
                del(parent[index])
        return leaves[0]._set_duration(total_preprolated)

    def _fuse_measures(self):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # check input
        prototype = (scoretools.Measure,)
        assert self._all_are_contiguous_components_in_same_parent(
            self, prototype)
        # return none on empty measures
        if len(self) == 0:
            return None
        # TODO: instantiate a new measure
        #       instead of returning a reference to existing measure
        if len(self) == 1:
            return self[0]
        implicit_scaling = self[0].implicit_scaling
        assert all(
            x.implicit_scaling == implicit_scaling for x in self)
        selection = selectiontools.Selection(self)
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        old_denominators = []
        new_duration = durationtools.Duration(0)
        for measure in self:
            effective_time_signature = measure.time_signature
            old_denominators.append(effective_time_signature.denominator)
            new_duration += effective_time_signature.duration
        new_time_signature = \
            measure._duration_and_possible_denominators_to_time_signature(
                new_duration,
                old_denominators,
                )
        music = []
        for measure in self:
            # scale before reassignment to prevent logical tie scale drama
            signature = measure.time_signature
            prolation = signature.implied_prolation
            multiplier = prolation / new_time_signature.implied_prolation
            measure._scale_contents(multiplier)
            measure_music = measure[:]
            measure_music._set_parents(None)
            music += measure_music
        new_measure = scoretools.Measure(new_time_signature, music)
        new_measure.implicit_scaling = self[0].implicit_scaling
        if parent is not None:
            self._give_dominant_spanners([new_measure])
        self._set_parents(None)
        if parent is not None:
            parent.insert(start, new_measure)
        return new_measure

    def _fuse_tuplets(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_parent(
            self, prototype=(scoretools.Tuplet,))
        if len(self) == 0:
            return None
        first = self[0]
        first_multiplier = first.multiplier
        first_type = type(first)
        for tuplet in self[1:]:
            if tuplet.multiplier != first_multiplier:
                message = 'tuplets must carry same multiplier.'
                raise ValueError(message)
            if type(tuplet) != first_type:
                message = 'tuplets must be same type.'
                raise TypeError(message)
        if isinstance(first, scoretools.FixedDurationTuplet):
            total_contents_duration = sum(
                [x._contents_duration for x in self])
            new_target_duration = first_multiplier * total_contents_duration
            new_tuplet = scoretools.FixedDurationTuplet(
                new_target_duration, [])
        elif isinstance(first, scoretools.Tuplet):
            new_tuplet = scoretools.Tuplet(first_multiplier, [])
        else:
            message = 'unknown tuplet type.'
            raise TypeError(message)
        wrapped = False
        if (self[0]._get_parentage().root is not
            self[-1]._get_parentage().root):
            dummy_container = scoretools.Container(self)
            wrapped = True
        mutate(self).swap(new_tuplet)
        if wrapped:
            del(dummy_container[:])
        return new_tuplet

    def _get_component(self, prototype=None, n=0, recurse=True):
        from abjad.tools import scoretools
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        if 0 <= n:
            if recurse:
                components = iterate(self).by_class(prototype)
            else:
                components = self._music
            for i, x in enumerate(components):
                if i == n:
                    return x
        else:
            if recurse:
                components = iterate(self).by_class(
                    prototype, reverse=True)
            else:
                components = reversed(self._music)
            for i, x in enumerate(components):
                if i == abs(n) - 1:
                    return x

    def _get_crossing_spanners(self):
        r'''Assert logical-voice-contiguous components.
        Collect spanners that attach to any component in selection.
        Returns unordered set of crossing spanners.
        A spanner P crosses a list of logical-voice-contiguous components C
        when P and C share at least one component and when it is the
        case that NOT ALL of the components in P are also in C.
        In other words, there is some intersection -- but not total
        intersection -- between the components of P and C.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        all_components = set(iterate(self).by_class())
        contained_spanners = set()
        for component in iterate(self).by_class():
            contained_spanners.update(component._get_spanners())
        crossing_spanners = set([])
        for spanner in contained_spanners:
            spanner_components = set(spanner[:])
            if not spanner_components.issubset(all_components):
                crossing_spanners.add(spanner)
        return crossing_spanners

    def _get_dominant_spanners(self):
        r'''Returns spanners that dominate components in selection.
        Returns set of (spanner, index) pairs.
        Each (spanner, index) pair gives a spanner which dominates
        all components in selection together with the start index
        at which spanner first encounters selection.
        Use this helper to lift spanners temporarily from components
        in selection and perform some action to the underlying
        score tree before reattaching spanners.
        score components.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        receipt = set([])
        if len(self) == 0:
            return receipt
        first, last = self[0], self[-1]
        start_components = first._get_descendants_starting_with()
        stop_components = last._get_descendants_stopping_with()
        stop_components = set(stop_components)
        for component in start_components:
            for spanner in component._get_spanners():
                if set(spanner[:]) & stop_components != set([]):
                    index = spanner._index(component)
                    receipt.add((spanner, index))
        return receipt

    def _get_format_specification(self):
        values = []
        if self._music:
            values = [list(self._music)]
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            )

    def _get_offset_lists(self):
        start_offsets, stop_offsets = [], []
        for component in self:
            start_offsets.append(component._get_timespan().start_offset)
            stop_offsets.append(component._get_timespan().stop_offset)
        return start_offsets, stop_offsets

    def _get_parent_and_start_stop_indices(self):
        assert self._all_are_contiguous_components_in_same_parent(self)
        if self:
            first, last = self[0], self[-1]
            parent = first._parent
            if parent is not None:
                first_index = parent.index(first)
                last_index = parent.index(last)
                return parent, first_index, last_index
        return None, None, None

    def _get_spanner(self, prototype=None):
        spanners = self._get_spanners(prototype=prototype)
        if not spanners:
            message = 'no spanners found.'
            raise MissingSpannerError(message)
        elif len(spanners) == 1:
            return spanners.pop()
        else:
            message = 'multiple spanners found.'
            raise ExtraSpannerError(message)

    def _get_spanners(self, prototype=None):
        from abjad.tools import spannertools
        prototype = prototype or (spannertools.Spanner,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        result = set()
        for component in self:
            spanners = component._get_spanners(prototype)
            result.update(spanners)
        return result

    def _give_dominant_spanners(self, recipients):
        r'''Find all spanners dominating music.
        Insert each component in recipients into each dominant spanner.
        Remove music from each dominating spanner.
        Returns none.
        Not composer-safe.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        assert self._all_are_contiguous_components_in_same_logical_voice(
            recipients)
        receipt = self._get_dominant_spanners()
        for spanner, index in receipt:
            for recipient in reversed(recipients):
                spanner._insert(index, recipient)
            for component in self:
                spanner._remove(component)

    def _give_music_to_empty_container(self, container):
        r'''Not composer-safe.
        '''
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_parent(self)
        assert isinstance(container, scoretools.Container)
        assert not container
        music = []
        for component in self:
            music.extend(getattr(component, '_music', ()))
        container._music.extend(music)
        container[:]._set_parents(container)

    def _give_position_in_parent_to_container(self, container):
        r'''Not composer-safe.
        '''
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_parent(self)
        assert isinstance(container, scoretools.Container)
        parent, start, stop = self._get_parent_and_start_stop_indices()
        if parent is not None:
            parent._music.__setitem__(slice(start, start), [container])
            container._set_parent(parent)
            self._set_parents(None)

    def _iterate_components(self, recurse=True, reverse=False):
        if recurse:
            return iterate(self).by_class()
        else:
            return self._iterate_top_level_components(reverse=reverse)

    def _iterate_top_level_components(self, reverse=False):
        if reverse:
            for component in reversed(self):
                yield component
        else:
            for component in self:
                yield component

    def _make_spanner_schema(self):
        schema = {}
        spanners = set()
        for component in iterate(self).by_class():
            spanners.update(component._get_spanners())
        for spanner in spanners:
            schema[spanner] = []
        for i, component in \
            enumerate(iterate(self).by_class()):
            attached_spanners = component._get_spanners()
            for attached_spanner in attached_spanners:
                try:
                    schema[attached_spanner].append(i)
                except KeyError:
                    pass
        return schema

    def _set_parents(self, new_parent):
        r'''Not composer-safe.
        '''
        for component in self._music:
            component._set_parent(new_parent)

    def _withdraw_from_crossing_spanners(self):
        r'''Not composer-safe.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        crossing_spanners = self._get_crossing_spanners()
        components_including_children = select(self).by_class()
        for crossing_spanner in list(crossing_spanners):
            spanner_components = crossing_spanner._components[:]
            for component in components_including_children:
                if component in spanner_components:
                    crossing_spanner._components.remove(component)
                    component._spanners.discard(crossing_spanner)

    ### PUBLIC METHODS ###

    def by_class(
        self,
        prototype=None,
        reverse=False,
        start=0,
        stop=None,
        with_grace_notes=False,
        ):
        r'''Select components by class.

        ..  container:: example

            ::

                >>> staff = Staff()
                >>> staff.append(Measure((2, 8), "c'8 d'8"))
                >>> staff.append(Measure((2, 8), "e'8 f'8"))
                >>> staff.append(Measure((2, 8), "g'8 a'8"))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    {
                        \time 2/8
                        c'8
                        d'8
                    }
                    {
                        e'8
                        f'8
                    }
                    {
                        g'8
                        a'8
                    }
                }

            ::

                >>> for note in select(staff).by_class(prototype=Note):
                ...     note
                ...
                Note("c'8")
                Note("d'8")
                Note("e'8")
                Note("f'8")
                Note("g'8")
                Note("a'8")

        Returns new selection.
        '''
        iterator = iterate(self).by_class(
            prototype=prototype,
            reverse=reverse,
            start=start,
            stop=stop,
            with_grace_notes=with_grace_notes,
            )
        return Selection(iterator)

    def by_leaf(
        self,
        prototype=None,
        reverse=False,
        start=0,
        stop=None,
        with_grace_notes=False,
        ):
        r'''Select components by leaf.

        ..  container:: example

            ::

                >>> staff = Staff()
                >>> staff.append(Measure((2, 8), "<c' bf'>8 <g' a'>8"))
                >>> staff.append(Measure((2, 8), "af'8 r8"))
                >>> staff.append(Measure((2, 8), "r8 gf'8"))
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    {
                        \time 2/8
                        <c' bf'>8
                        <g' a'>8
                    }
                    {
                        af'8
                        r8
                    }
                    {
                        r8
                        gf'8
                    }
                }

            ::

                >>> for leaf in select(staff).by_leaf():
                ...     leaf
                ...
                Chord("<c' bf'>8")
                Chord("<g' a'>8")
                Note("af'8")
                Rest('r8')
                Rest('r8')
                Note("gf'8")

        Returns new selection.
        '''
        iterator = iterate(self).by_leaf(
            prototype=prototype,
            reverse=reverse,
            start=start,
            stop=stop,
            with_grace_notes=with_grace_notes,
            )
        return Selection(iterator)

    def by_logical_tie(
        self,
        nontrivial=False,
        pitched=False,
        reverse=False,
        parentage_mask=None,
        with_grace_notes=False,
        ):
        r'''Select components by logical tie.

        ..  container:: example

            ::

                >>> staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'4 ~
                    \times 2/3 {
                        c'16
                        d'8
                    }
                    e'8
                    f'4 ~
                    f'16
                }

            ::

                >>> for logical_tie in select(staff).by_logical_tie():
                ...     logical_tie
                ...
                LogicalTie([Note("c'4"), Note("c'16")])
                LogicalTie([Note("d'8")])
                LogicalTie([Note("e'8")])
                LogicalTie([Note("f'4"), Note("f'16")])

        Returns new selection.
        '''
        iterator = iterate(self).by_logical_tie(
            nontrivial=nontrivial,
            pitched=pitched,
            reverse=reverse,
            parentage_mask=parentage_mask,
            with_grace_notes=with_grace_notes,
            )
        return Selection(iterator)

    def by_run(self, prototype=None):
        r'''Select components by run.

        ..  container:: example

            ::

                >>> staff = Staff(r"\times 2/3 { c'8 d'8 r8 }")
                >>> staff.append(r"\times 2/3 { r8 <e' g'>8 <f' a'>8 }")
                >>> staff.extend("g'8 a'8 r8 r8 <b' d''>8 <c'' e''>8")
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \times 2/3 {
                        c'8
                        d'8
                        r8
                    }
                    \times 2/3 {
                        r8
                        <e' g'>8
                        <f' a'>8
                    }
                    g'8
                    a'8
                    r8
                    r8
                    <b' d''>8
                    <c'' e''>8
                }

            ::

                >>> for group in select(staff[:]).by_run((Note, Chord)):
                ...     group
                ...
                Selection([Note("g'8"), Note("a'8")])
                Selection([Chord("<b' d''>8"), Chord("<c'' e''>8")])

        Returns new selection.
        '''
        iterator = iterate(self).by_run(prototype=prototype)
        return Selection(iterator)

    def by_timeline(self, prototype=None, reverse=False):
        r'''Select components by timeline.

        ..  container:: example

            ::

                >>> score = Score([])
                >>> score.append(Staff("c'4 d'4 e'4 f'4"))
                >>> score.append(Staff("g'8 a'8 b'8 c''8"))
                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score <<
                    \new Staff {
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                    \new Staff {
                        g'8
                        a'8
                        b'8
                        c''8
                    }
                >>

            ::

                >>> for leaf in select(score).by_timeline():
                ...     leaf
                ...
                Note("c'4")
                Note("g'8")
                Note("a'8")
                Note("d'4")
                Note("b'8")
                Note("c''8")
                Note("e'4")
                Note("f'4")


        Returns new selection.
        '''
        iterator = iterate(self).by_timeline(
            prototype=prototype,
            reverse=reverse,
            )
        return Selection(iterator)

    def by_timeline_and_logical_tie(
        self,
        nontrivial=False,
        pitched=False,
        reverse=False,
        ):
        r'''Select components by timeline and logical tie.

        ..  container:: example

            ::

                >>> score = Score([])
                >>> score.append(Staff("c''4 ~ c''8 d''8 r4 ef''4"))
                >>> score.append(Staff("r8 g'4. ~ g'8 r16 f'8. ~ f'8"))
                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score <<
                    \new Staff {
                        c''4 ~
                        c''8
                        d''8
                        r4
                        ef''4
                    }
                    \new Staff {
                        r8
                        g'4. ~
                        g'8
                        r16
                        f'8. ~
                        f'8
                    }
                >>

            ::

                >>> for logical_tie in select(score).by_timeline_and_logical_tie():
                ...     logical_tie
                ...
                LogicalTie([Note("c''4"), Note("c''8")])
                LogicalTie([Rest('r8')])
                LogicalTie([Note("g'4."), Note("g'8")])
                LogicalTie([Note("d''8")])
                LogicalTie([Rest('r4')])
                LogicalTie([Rest('r16')])
                LogicalTie([Note("f'8."), Note("f'8")])
                LogicalTie([Note("ef''4")])

        Returns new selection.
        '''
        iterator = iterate(self).by_timeline_and_logical_tie(
            nontrivial=nontrivial,
            pitched=pitched,
            reverse=reverse,
            )
        return Selection(iterator)

    def get_duration(self, in_seconds=False):
        r'''Gets duration of contiguous selection.

        Returns duration.
        '''
        durations = []
        for element in self:
            if hasattr(element, '_get_duration'):
                duration = element._get_duration(in_seconds=in_seconds)
            else:
                duration = durationtools.Duration(element)
            durations.append(duration)
        return sum(durations)

    def get_spanners(self, prototype=None, in_parentage=False):
        r'''Gets spanners attached to any component in selection.

        Returns set.
        '''
        result = set()
        for component in self:
            spanners = component._get_spanners(
                prototype=prototype,
                in_parentage=in_parentage,
                )
            result.update(spanners)
        return result

    def get_timespan(self, in_seconds=False):
        r'''Gets timespan of contiguous selection.

        Returns timespan.
        '''
        from abjad.tools import timespantools
        if in_seconds:
            raise NotImplementedError
        timespan = self[0]._get_timespan()
        start_offset = timespan.start_offset
        stop_offset = timespan.stop_offset
        for x in self[1:]:
            timespan = x._get_timespan()
            if timespan.start_offset < start_offset:
                start_offset = timespan.start_offset
            if stop_offset < timespan.stop_offset:
                stop_offset = timespan.stop_offset
        return timespantools.Timespan(start_offset, stop_offset)

    def get_vertical_moment_at(self, offset):
        r'''Select vertical moment at `offset`.
        '''
        from abjad.tools import selectiontools
        return selectiontools.VerticalMoment(self, offset)

    def group_by(self, predicate):
        '''Groups components in contiguous selection by `predicate`.

        Returns list of tuples.
        '''
        result = []
        grouper = itertools.groupby(self, predicate)
        for label, generator in grouper:
            selection = tuple(generator)
            result.append(selection)
        return result

    def partition_by_durations(
        self,
        durations,
        cyclic=False,
        fill=Exact,
        in_seconds=False,
        overhang=False,
        ):
        r'''Partitions `components` according to `durations`.

        When `fill` is `Exact` then parts must equal `durations` exactly.

        When `fill` is `Less` then parts must be
        less than or equal to `durations`.

        When `fill` is `More` then parts must be
        greater or equal to `durations`.

        Reads `durations` cyclically when `cyclic` is true.

        Reads component durations in seconds when `in_seconds` is true.

        Returns remaining components at end in final part when `overhang`
        is true.
        '''
        durations = [durationtools.Duration(x) for x in durations]
        if cyclic:
            durations = datastructuretools.CyclicTuple(durations)
        result = []
        part = []
        current_duration_index = 0
        target_duration = durations[current_duration_index]
        cumulative_duration = durationtools.Duration(0)
        components_copy = list(self)
        while True:
            try:
                component = components_copy.pop(0)
            except IndexError:
                break
            component_duration = component._get_duration()
            if in_seconds:
                component_duration = component._get_duration(in_seconds=True)
            candidate_duration = cumulative_duration + component_duration
            if candidate_duration < target_duration:
                part.append(component)
                cumulative_duration = candidate_duration
            elif candidate_duration == target_duration:
                part.append(component)
                result.append(part)
                part = []
                cumulative_duration = durationtools.Duration(0)
                current_duration_index += 1
                try:
                    target_duration = durations[current_duration_index]
                except IndexError:
                    break
            elif target_duration < candidate_duration:
                if fill == Exact:
                    message = 'must partition exactly.'
                    raise Exception(message)
                elif fill == Less:
                    result.append(part)
                    part = [component]
                    if in_seconds:
                        cumulative_duration = \
                            sum([x._get_duration(in_seconds=True)
                            for x in part])
                    else:
                        cumulative_duration = \
                            sum([x._get_duration() for x in part])
                    current_duration_index += 1
                    try:
                        target_duration = durations[current_duration_index]
                    except IndexError:
                        break
                    if target_duration < cumulative_duration:
                        message = 'target duration {}'
                        message += ' is less than cumulative duration {}.'
                        message = message.format(
                            target_duration,
                            cumulative_duration,
                            )
                        raise Exception(message)
                elif fill == More:
                    part.append(component)
                    result.append(part)
                    part = []
                    cumulative_duration = durationtools.Duration(0)
                    current_duration_index += 1
                    try:
                        target_duration = durations[current_duration_index]
                    except IndexError:
                        break
        if len(part):
            if overhang:
                result.append(part)
        if len(components_copy):
            if overhang:
                result.append(components_copy)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _preprolated_duration(self):
        return sum(component._preprolated_duration for component in self)

collections.Sequence.register(Selection)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordExtent___init___01():
    r'''Initialize from number.
    '''

    assert tonalanalysistools.ChordExtent(7).number == 7


def test_tonalanalysistools_ChordExtent___init___02():
    r'''Initialize by reference.
    '''

    chord_extent = tonalanalysistools.ChordExtent(7)
    new = tonalanalysistools.ChordExtent(chord_extent)

    assert new.number == 7
    assert new == chord_extent
    assert new is not chord_extent



# -*- coding: utf-8 -*-


def is_n_tuple(argument, n):
    r'''Is true when `argument` is a tuple of length `n`. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_n_tuple((19, 20, 21), 3)
            True

        ::

            >>> mathtools.is_n_tuple((19, 20, 21), 4)
            False

    Returns true or false.
    '''
    return isinstance(argument, tuple) and len(argument) == n



from abjad import *


def test_rhythmmakertools_DurationSpellingSpecifier_01():
    r'''DurationSpellingSpecifier does not leave parent references.'''

    beam_specifier = rhythmmakertools.BeamSpecifier(
        beam_each_division=True,
        beam_divisions_together=False,
        use_feather_beams=False,
        )

    duration_spelling_specifier = rhythmmakertools.DurationSpellingSpecifier(
        decrease_durations_monotonically=True,
        forbid_meter_rewriting=False,
        rewrite_meter=True,
        spell_metrically='unassignable',
        )

    logical_tie_masks = [rhythmmakertools.silence_every([0], period=2)]

    talea = rhythmmakertools.Talea(counts=[1, 2, 3, 4], denominator=16)

    maker = rhythmmakertools.TaleaRhythmMaker(
        beam_specifier=beam_specifier,
        duration_spelling_specifier=duration_spelling_specifier,
        logical_tie_masks=logical_tie_masks,
        talea=talea,
        )

    divisions = [TimeSignature((3, 8)), TimeSignature((2, 4)), TimeSignature((5, 16))]

    result = maker(divisions)

    staff = Staff(result)

    assert format(staff) == stringtools.normalize(
        r"""
        \new Staff {
            r16
            c'16 ~ [
            c'16 ]
            r8.
            c'4
            r16
            c'16 ~ [
            c'16 ]
            r16
            r8
            c'8.
        }
        """)



# -*- coding: utf-8 -*-


def to_bidirectional_direction_string(argument):
    r'''Changes `argument` to bidirectional direction string.

    ..  container:: example:

        ::

            >>> stringtools.to_bidirectional_direction_string('^')
            'up'

    ..  container:: example:

        ::

            >>> stringtools.to_bidirectional_direction_string('_')
            'down'

    ..  container:: example:

        ::

            >>> stringtools.to_bidirectional_direction_string(1)
            'up'

    ..  container:: example:

        ::

            >>> stringtools.to_bidirectional_direction_string(-1)
            'down'

    Returns `argument` when `argument` is `'up'` or `'down'`.

    Returns string or none.
    '''
    lookup = {
        1: 'up',
        -1: 'down',
        Up: 'up',
        Down: 'down',
        '^': 'up',
        '_': 'down',
        'up': 'up',
        'down': 'down'
        }
    if argument in lookup:
        return lookup[argument]
    raise ValueError(argument)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


@pytest.mark.skip()
def test_scoretools_Measure___add___01():
    r'''Add outside-of-score measures.
    '''

    measure_1 = Measure((1, 8), "c'16 d'16")
    beam = Beam()
    attach(beam, measure_1[:])
    measure_2 = Measure((2, 16), "c'16 d'16")
    slur = Slur()
    attach(slur, measure_2[:])
    new = measure_1 + measure_2

    assert format(new) == stringtools.normalize(
        r'''
        {
            \time 2/8
            c'16 [
            d'16 ]
            c'16 (
            d'16 )
        }
        '''
        )

    assert new is not measure_1 and new is not measure_2
    assert len(measure_1) == 0
    assert len(measure_2) == 0
    assert inspect_(new).is_well_formed()


@pytest.mark.skip()
def test_scoretools_Measure___add___02():
    r'''Add measures in score.
    '''

    measure_1 = Measure((1, 8), "c'16 d'16")
    beam = Beam()
    attach(beam, measure_1[:])
    measure_2 = Measure((2, 16), "c'16 d'16")
    slur = Slur()
    attach(slur, measure_2[:])
    staff = Staff([measure_1, measure_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'16 [
                d'16 ]
            }
            {
                \time 2/16
                c'16 (
                d'16 )
            }
        }
        '''
        )

    new = measure_1 + measure_2

    assert new is not measure_1 and new is not measure_2
    assert len(measure_1) == 0
    assert len(measure_2) == 0

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'16 [
                d'16 ]
                c'16 (
                d'16 )
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHead_is_parenthesized_01():

    note_head = scoretools.NoteHead(written_pitch="c'")
    assert note_head.is_parenthesized is None
    note_head.is_parenthesized = True
    assert note_head.is_parenthesized == True
    note_head.is_parenthesized = False
    assert note_head.is_parenthesized == False


def test_scoretools_NoteHead_is_parenthesized_02():

    note_head = scoretools.NoteHead(written_pitch="c'")
    note_head.is_parenthesized = True
    assert format(note_head) == stringtools.normalize(
        r'''
        \parenthesize
        c'
        ''')


def test_scoretools_NoteHead_is_parenthesized_03():

    note = scoretools.Note("c'4")
    note.note_head.is_parenthesized = True
    assert format(note) == stringtools.normalize(
        r'''
        \parenthesize
        c'4
        ''')


def test_scoretools_NoteHead_is_parenthesized_04():

    chord = scoretools.Chord("<c' e' g'>4")
    chord.note_heads[1].is_parenthesized = True
    assert format(chord) == stringtools.normalize(
        r'''
        <
            c'
            \parenthesize
            e'
            g'
        >4
        ''')



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ScaleDegree__initialize_by_pair_01():

    degree = tonalanalysistools.ScaleDegree(('flat', 2))

    assert degree.accidental == pitchtools.Accidental('flat')
    assert degree.number == 2



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_StemTremolo___init___01():
    r'''Initializes stem tremolo from tremolo flags.
    '''

    stem_tremolo = indicatortools.StemTremolo(16)
    assert isinstance(stem_tremolo, indicatortools.StemTremolo)


def test_indicatortools_StemTremolo___init___02():
    r'''Initializes stem tremolo from other stem tremolo.
    '''

    stem_tremolo_1 = indicatortools.StemTremolo(16)
    stem_tremolo_2 = indicatortools.StemTremolo(stem_tremolo_1)

    assert isinstance(stem_tremolo_1, indicatortools.StemTremolo)
    assert isinstance(stem_tremolo_2, indicatortools.StemTremolo)
    assert stem_tremolo_1 == stem_tremolo_2
    assert stem_tremolo_1 is not stem_tremolo_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___sub___01():

    major_second_ascending = pitchtools.NamedInterval('major', 2)
    major_third_ascending = pitchtools.NamedInterval('major', 3)

    difference = major_second_ascending - major_third_ascending
    assert difference == pitchtools.NamedInterval('major', -2)

    difference = major_third_ascending - major_second_ascending
    assert difference == pitchtools.NamedInterval('major', 2)

    difference = major_second_ascending - major_second_ascending
    assert difference == pitchtools.NamedInterval('perfect', 1)

    difference = major_third_ascending - major_third_ascending
    assert difference == pitchtools.NamedInterval('perfect', 1)



# -*- coding: utf-8 -*-
import math
import numbers


def greatest_multiple_less_equal(m, n):
    r'''Gets greatest integer multiple of `m` less than or equal to `n`.

    ..  container:: example

        ::

            >>> mathtools.greatest_multiple_less_equal(10, 47)
            40

        ::

            >>> for m in range(1, 10):
            ...     print(m, mathtools.greatest_multiple_less_equal(m, 47))
            ... 
            1 47
            2 46
            3 45
            4 44
            5 45
            6 42
            7 42
            8 40
            9 45

        ::

            >>> for n in range(10, 100, 10):
            ...     print(mathtools.greatest_multiple_less_equal(7, n), n)
            ... 
            7 10
            14 20
            28 30
            35 40
            49 50
            56 60
            70 70
            77 80
            84 90

    Returns nonnegative integer.
    '''
    if not isinstance(m, numbers.Number):
        message = 'must be number: {!r}.'
        message = message.format(m)
        raise TypeError(message)
    if not isinstance(n, numbers.Number):
        message = 'must be number: {!r}.'
        message = message.format(n)
        raise TypeError(message)
    return m * int(math.floor(n / float(m)))



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment___add___01():
    r'''Adding pitch-class segments returns a new pitch-class segment.
    '''

    pitch_class_segment_1 = pitchtools.PitchClassSegment([0, 1, 2])
    pitch_class_segment_2 = pitchtools.PitchClassSegment([3, 4, 5])

    assert pitch_class_segment_1 + pitch_class_segment_2 == \
        pitchtools.PitchClassSegment([0, 1, 2, 3, 4, 5])



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class WellformednessManager(AbjadObject):
    r'''Wellformedness manager.

    ..  container:: example

        ::

            >>> systemtools.WellformednessManager()
            WellformednessManager()

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Managers'

    ### SPECIAL METHODS ###

    def __call__(self, argument=None):
        r'''Calls all wellformedness checks on `argument`.

        Returns triples.
        '''
        if argument is None:
            return
        check_names = [x for x in dir(self) if x.startswith('check_')]
        triples = []
        for current_check_name in sorted(check_names):
            current_check = getattr(self, current_check_name)
            current_violators, current_total = current_check(argument=argument)
            triple = (current_violators, current_total, current_check_name)
            triples.append(triple)
        return triples

    ### PUBLIC METHODS ###

    @staticmethod
    def check_beamed_quarter_notes(argument=None):
        r'''Checks to make sure there are no beamed quarter notes.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        smart_beams = (
            spannertools.DuratedComplexBeam,
            spannertools.MultipartBeam,
            )
        for leaf in iterate(argument).by_leaf():
            total += 1
            parentage = leaf._get_parentage(include_self=True)
            beams = parentage._get_spanners(spannertools.Beam)
            for beam in beams:
                if not isinstance(beam, smart_beams):
                    flag_count = leaf.written_duration.flag_count
                    if flag_count < 1:
                        violators.append(leaf)
                        break
        return violators, total

    @staticmethod
    def check_conflicting_clefs(argument=None):
        r'''Checks for conflicting clefs.

        Conflicting clefs defined equal to the situation in which a first clef
        is attached to a container and a second clef is attached to a child of
        the container that starts at the same time as the container.

        Situation does not usually arise because an exception raises on attempt
        to attach a clef to any component that starts at the same time as some
        other component in the score tree.

        But advanced users can stumble into this situation as described in
        the following examples.

        ..  container:: example

            Conflicting clefs result from attaching clefs to separate
            containers and then appending one container to the other:

            ::

                >>> staff = Staff()
                >>> attach(Clef('bass'), staff)
                >>> voice = Voice()
                >>> attach(Clef('treble'), voice)
                >>> staff.append(voice)

            ::

                >>> f(staff)
                \new Staff {
                    \clef "bass"
                    \new Voice {
                        \clef "treble"
                    }
                }

            ::

                >>> inspect_(staff).is_well_formed()
                False

            This is bad.

        ..  container:: example

            Conflicting clefs result from attaching clefs to free-standing
            leaves, then to a container and then extending the
            cleffed-container with the cleffed-leaves:

            ::

                >>> staff = Staff()
                >>> attach(Clef('bass'), staff)
                >>> leaves = scoretools.make_leaves([0, 2, 4, 5], [(1, 4)])
                >>> attach(Clef('treble'), leaves[0])
                >>> staff.extend(leaves)

            ::

                >>> f(staff)
                \new Staff {
                    \clef "bass"
                    \clef "treble"
                    c'4
                    d'4
                    e'4
                    f'4
                }

            ::

                >>> inspect_(staff).is_well_formed()
                False

            This is also bad.

        ..  todo:: Raise an exception on append or extend to prohibit these
            cases.

        Returns violators and total.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import inspect_
        from abjad.tools.topleveltools import iterate
        violators = []
        containers = iterate(argument).by_class(scoretools.Container)
        total_containers = 0
        for container in containers:
            total_containers += 1
            if not inspect_(container).has_indicator(indicatortools.Clef):
                continue
            current_component = container
            while (isinstance(current_component, scoretools.Container) and
                0 < len(current_component)):
                first_child = current_component[0]
                if inspect_(first_child).has_indicator(indicatortools.Clef):
                    violators.append(container)
                    break
                current_component = first_child
        return violators, total_containers

    @staticmethod
    def check_discontiguous_spanners(argument=None):
        r'''Checks for discontiguous spanners.

        There are now two different types of spanner.
        Most spanners demand that spanner components be
        logical-voice-contiguous. But a few special spanners (like Tempo)
        do not make such a demand. The check here consults the experimental
        `_contiguity_constraint`.

        Returns violators and total.
        '''
        from abjad.tools.selectiontools import Selection
        violators = []
        total = 0
        for spanner in argument._get_descendants()._get_spanners():
            if spanner._contiguity_constraint == 'logical voice':
                if not Selection._all_are_contiguous_components_in_same_logical_voice(
                    spanner[:]):
                    violators.append(spanner)
            total += 1
        return violators, total

    @staticmethod
    def check_duplicate_ids(argument=None):
        r'''Checks to make sure there are no components with duplicated IDs.

        Returns violators and total.
        '''
        from abjad.tools import sequencetools
        from abjad.tools.topleveltools import iterate
        violators = []
        components = iterate(argument).by_class()
        total_ids = [id(x) for x in components]
        unique_ids = sequencetools.Sequence(total_ids).remove_repeats()
        if len(unique_ids) < len(total_ids):
            for current_id in unique_ids:
                if 1 < total_ids.count(current_id):
                    violators.extend([x for x in components
                        if id(x) == current_id])
        return violators, len(total_ids)

    @staticmethod
    def check_empty_containers(argument=None):
        r'''Checks to make sure there are no empty containers in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        bad, total = 0, 0
        for component in iterate(argument).by_class(scoretools.Container):
            if len(component) == 0:
                violators.append(component)
                bad += 1
            total += 1
        return violators, total

    @staticmethod
    def check_intermarked_hairpins(argument=None):
        r'''Checks to make sure there are no hairpins in score with intervening
        dynamic marks.

        Returns violators and total.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import spannertools
        violators = []
        total, bad = 0, 0
        prototype = (spannertools.Hairpin,)
        hairpins = argument._get_descendants()._get_spanners(prototype)
        for hairpin in hairpins:
            if 2 < len(hairpin._get_leaves()):
                for leaf in hairpin._get_leaves()[1:-1]:
                    if leaf._get_indicators(indicatortools.Dynamic):
                        violators.append(hairpin)
                        bad += 1
                        break
            total += 1
        return violators, total

    @staticmethod
    def check_misdurated_measures(argument=None):
        r'''Checks to make sure there are no misdurated measures in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        total, bad = 0, 0
        for measure in iterate(argument).by_class(scoretools.Measure):
            time_signature = measure.time_signature
            if time_signature is not None:
                if measure._preprolated_duration != time_signature.duration:
                    violators.append(measure)
                    bad += 1
            total += 1
        return violators, total

    @staticmethod
    def check_misfilled_measures(argument=None):
        r'''Checks that time signature duration equals measure contents
        duration for every measure.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        total, bad = 0, 0
        for measure in iterate(argument).by_class(scoretools.Measure):
            if measure.is_misfilled:
                violators.append(measure)
                bad += 1
            total += 1
        return violators, total

    @staticmethod
    def check_mismatched_enchained_hairpins(argument=None):
        r'''Checks mismatched enchained hairpins.

        ..  container:: example

            Checks mismatched enchained hairpins:

            ::

                >>> staff = Staff("c'4 d' e' f'")
                >>> attach(Hairpin('p < f'), staff[:2])
                >>> attach(Hairpin('p > pp'), staff[1:])

            ::

                >>> print(inspect_(staff).tabulate_well_formedness_violations())
                0 /	4 beamed quarter notes
                0 /	1 conflicting clefs
                0 /	2 discontiguous spanners
                0 /	5 duplicate ids
                0 /     1 empty containers
                0 /	2 intermarked hairpins
                0 /	0 misdurated measures
                0 /	0 misfilled measures
                2 /	2 mismatched enchained hairpins
                0 /	0 mispitched ties
                0 /	4 misrepresented flags
                0 /	5 missing parents
                0 /	0 nested measures
                0 /	0 overlapping beams
                0 /	0 overlapping glissandi
                0 /	2 overlapping hairpins
                0 /	0 overlapping octavation spanners
                0 /	0 overlapping ties
                0 /	2 short hairpins
                0 /	0 tied rests

        Enchained hairpins are fine so long as hairpin ends match.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        all_hairpins = set()
        prototype = spannertools.Hairpin
        for leaf in iterate(argument).by_leaf():
            hairpins = leaf._get_spanners(prototype)
            hairpins = list(hairpins)
            all_hairpins.update(hairpins)
            if len(hairpins) <= 1:
                continue
            if 2 < len(hairpins):
                raise Exception('too many hairpins')
            assert len(hairpins) == 2
            hairpins_are_enchained = False
            if (hairpins[0]._is_my_last_leaf(leaf) and 
                hairpins[-1]._is_my_first_leaf(leaf)):
                hairpins_are_enchained = True
            if (hairpins[-1]._is_my_last_leaf(leaf) and 
                hairpins[0]._is_my_first_leaf(leaf)):
                hairpins_are_enchained = True
            if not hairpins_are_enchained:
                continue
            if hairpins[0]._is_my_first_leaf(leaf):
                first_hairpin = hairpins[-1]
                second_hairpin = hairpins[0]
            else:
                first_hairpin = hairpins[0]
                second_hairpin = hairpins[-1]
            if first_hairpin.stop_dynamic == second_hairpin.start_dynamic:
                continue
            else:
                violators.append(first_hairpin)
                violators.append(second_hairpin)
        total = len(all_hairpins)
        return violators, total

    @staticmethod
    def check_mispitched_ties(argument=None):
        r'''Checks for mispitched notes.

        ..  container:: example

            Checks for mispitched ties attached to notes:

            ::

                >>> staff = Staff("c'4 ~ c'")
                >>> staff[1].written_pitch = NamedPitch("d'")

            ::

                >>> inspector = inspect_(staff)
                >>> print(inspector.tabulate_well_formedness_violations())
                0 /	2 beamed quarter notes
                0 /	1 conflicting clefs
                0 /	1 discontiguous spanners
                0 /	3 duplicate ids
                0 /     1 empty containers
                0 /	0 intermarked hairpins
                0 /	0 misdurated measures
                0 /	0 misfilled measures
                0 /	0 mismatched enchained hairpins
                1 /	1 mispitched ties
                0 /	2 misrepresented flags
                0 /	3 missing parents
                0 /	0 nested measures
                0 /	0 overlapping beams
                0 /	0 overlapping glissandi
                0 /	0 overlapping hairpins
                0 /	0 overlapping octavation spanners
                0 /	1 overlapping ties
                0 /	0 short hairpins
                0 /	0 tied rests

        ..  container:: example

            Checks for mispitched ties attached to chords:

            ::

                >>> staff = Staff("<c' d' bf'>4 ~ <c' d' bf'>")
                >>> staff[1].written_pitches = [6, 9, 10]

            ::

                >>> inspector = inspect_(staff)
                >>> print(inspector.tabulate_well_formedness_violations())
                0 /	2 beamed quarter notes
                0 /	1 conflicting clefs
                0 /	1 discontiguous spanners
                0 /	3 duplicate ids
                0 /     1 empty containers
                0 /	0 intermarked hairpins
                0 /	0 misdurated measures
                0 /	0 misfilled measures
                0 /	0 mismatched enchained hairpins
                1 /	1 mispitched ties
                0 /	2 misrepresented flags
                0 /	3 missing parents
                0 /	0 nested measures
                0 /	0 overlapping beams
                0 /	0 overlapping glissandi
                0 /	0 overlapping hairpins
                0 /	0 overlapping octavation spanners
                0 /	1 overlapping ties
                0 /	0 short hairpins
                0 /	0 tied rests
        
        Does not check tied rests, chords or skips.

        Returns violator ties together with total number of ties.
        '''
        from abjad.tools import mathtools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = set()
        all_spanners = set()
        pitched_prototype = (scoretools.Note, scoretools.Chord)
        for leaf in iterate(argument).by_class(pitched_prototype):
            spanners = leaf._get_spanners(spannertools.Tie)
            if not spanners:
                continue
            all_spanners.update(spanners)
            spanner = spanners.pop()
            written_pitches = []
            for leaf in spanner:
                if isinstance(leaf, scoretools.Note):
                    written_pitches.append(leaf.written_pitch)
                elif isinstance(leaf, scoretools.Chord):
                    written_pitches.append(leaf.written_pitches)
                else:
                    raise TypeError(leaf)
            if not mathtools.all_are_equal(written_pitches):
                violators.add(spanner)
        violators = list(violators)
        total = len(all_spanners)
        return violators, total

    @staticmethod
    def check_misrepresented_flags(argument=None):
        r'''Checks to make sure there are no misrepresented flags in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        from abjad.tools.topleveltools import set_
        violators = []
        total = 0
        for leaf in iterate(argument).by_leaf():
            total += 1
            flags = leaf.written_duration.flag_count
            left = getattr(set_(leaf), 'stem_left_beam_count', None)
            right = getattr(set_(leaf), 'stem_right_beam_count', None)
            if left is not None:
                if (flags < left or
                    (left < flags and right not in (flags, None))):
                    if leaf not in violators:
                        violators.append(leaf)
            if right is not None:
                if (flags < right or
                    (right < flags and left not in (flags, None))):
                    if leaf not in violators:
                        violators.append(leaf)
        return violators, total

    @staticmethod
    def check_missing_parents(argument=None):
        r'''Checks to make sure there are no components in score with missing
        parent.

        Returns violators and total.
        '''
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        components = iterate(argument).by_class()
        for i, component in enumerate(components):
            if 0 < i:
                if component._parent is None:
                    violators.append(component)
        total = i + 1
        return violators, total

    @staticmethod
    def check_nested_measures(argument=None):
        r'''Checks to make sure there are no nested measures in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        for measure in iterate(argument).by_class(scoretools.Measure):
            parentage = measure._get_parentage(include_self=False)
            if parentage.get_first(scoretools.Measure):
                violators.append(measure)
            total += 1
        return violators, total

    @staticmethod
    def check_overlapping_beams(argument=None):
        r'''Checks to make sure there are no overlapping beams in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = (spannertools.Beam,)
        all_beams = set()
        for leaf in iterate(argument).by_leaf():
            beams = leaf._get_spanners(prototype)
            all_beams.update(beams)
            if 1 < len(beams):
                for beam in beams:
                    if beam not in violators:
                        violators.append(beam)
        total = len(all_beams)
        return violators, total

    @staticmethod
    def check_overlapping_glissandi(argument=None):
        r'''Checks to make sure there are no overlapping glissandi in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = (spannertools.Glissando,)
        for leaf in iterate(argument).by_leaf():
            glissandi = leaf._get_spanners(prototype)
            glissandi = list(glissandi)
            if 1 < len(glissandi):
                if len(glissandi) == 2:
                    common_leaves = set(glissandi[0]._get_leaves())
                    common_leaves &= set(glissandi[1]._get_leaves())
                    if len(common_leaves) == 1:
                        x = list(common_leaves)[0]
                        if (
                            (glissandi[0]._is_my_first_leaf(x) and
                            glissandi[1]._is_my_last_leaf(x))
                            or
                            (glissandi[1]._is_my_first_leaf(x) and
                            glissandi[0]._is_my_last_leaf(x))
                            ):
                            break
                for glissando in glissandi:
                    if glissando not in violators:
                        violators.append(glissando)
        total = argument._get_descendants()._get_spanners(prototype)
        total = len(total)
        return violators, total

    @staticmethod
    def check_overlapping_hairpins(argument=None):
        r'''Checks to make sure there are no overlapping hairpins in score.

        ..  container:: example

            Checks overlapping hairpins:

            ::

                >>> staff = Staff("c'4 d' e' f'")
                >>> attach(Crescendo(), staff[:])
                >>> attach(Decrescendo(), staff[:])

            ::

                >>> print(inspect_(staff).tabulate_well_formedness_violations())
                0 /	4 beamed quarter notes
                0 /	1 conflicting clefs
                0 /	2 discontiguous spanners
                0 /	5 duplicate ids
                0 /     1 empty containers
                0 /	2 intermarked hairpins
                0 /	0 misdurated measures
                0 /	0 misfilled measures
                0 /	2 mismatched enchained hairpins
                0 /	0 mispitched ties
                0 /	4 misrepresented flags
                0 /	5 missing parents
                0 /	0 nested measures
                0 /	0 overlapping beams
                0 /	0 overlapping glissandi
                2 /	2 overlapping hairpins
                0 /	0 overlapping octavation spanners
                0 /	0 overlapping ties
                0 /	2 short hairpins
                0 /	0 tied rests

        Enchained hairpins are fine so long as hairpin ends match.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = spannertools.Hairpin
        for leaf in iterate(argument).by_leaf():
            hairpins = leaf._get_spanners(prototype)
            hairpins = list(hairpins)
            if 1 < len(hairpins):
                if len(hairpins) == 2:
                    common_leaves = set(hairpins[0]._get_leaves())
                    common_leaves &= set(hairpins[1]._get_leaves())
                    if len(common_leaves) == 1:
                        x = list(common_leaves)[0]
                        if (
                            (hairpins[0]._is_my_first_leaf(x) and
                            hairpins[1]._is_my_last_leaf(x))
                            or
                            (hairpins[1]._is_my_first_leaf(x) and
                            hairpins[0]._is_my_last_leaf(x))
                            ):
                            break
                for hairpin in hairpins:
                    if hairpin not in violators:
                        violators.append(hairpin)
        total = argument._get_descendants()._get_spanners(prototype)
        total = len(total)
        return violators, total

    @staticmethod
    def check_overlapping_octavation_spanners(argument=None):
        r'''Checks to make sure there are no overlapping octavation spanners in
        score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = (spannertools.OctavationSpanner, )
        for leaf in iterate(argument).by_leaf():
            spanners = leaf._get_descendants()._get_spanners(prototype)
            if 1 < len(spanners):
                for spanner in spanners:
                    if spanner not in violators:
                        violators.append(spanner)
        total = argument._get_descendants()._get_spanners(prototype)
        return violators, len(total)

    @staticmethod
    def check_overlapping_ties(argument=None):
        r'''Checks to make sure there are no overlapping ties in score.

        ..  container:: example

            Checks overlapping ties:

            ::

                >>> staff = Staff("c'4 c' c' c''")
                >>> attach(Tie(), staff[:2])
                >>> attach(Tie(), staff[1:3])

            ::

                >>> print(inspect_(staff).tabulate_well_formedness_violations())
                0 /	4 beamed quarter notes
                0 /	1 conflicting clefs
                0 /	2 discontiguous spanners
                0 /	5 duplicate ids
                0 /     1 empty containers
                0 /	0 intermarked hairpins
                0 /	0 misdurated measures
                0 /	0 misfilled measures
                0 /	0 mismatched enchained hairpins
                0 /	2 mispitched ties
                0 /	4 misrepresented flags
                0 /	5 missing parents
                0 /	0 nested measures
                0 /	0 overlapping beams
                0 /	0 overlapping glissandi
                0 /	0 overlapping hairpins
                0 /	0 overlapping octavation spanners
                2 /	2 overlapping ties
                0 /	0 short hairpins
                0 /	0 tied rests

        Returns violators and count of total ties.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        total = set()
        violators = set()
        for leaf in iterate(argument).by_leaf():
            spanners = leaf._get_spanners(spannertools.Tie)
            total.update(spanners)
            if 1 < len(spanners):
                violators.update(spanners)
        return violators, len(total)

    @staticmethod
    def check_short_hairpins(argument=None):
        r'''Checks to make sure that hairpins span at least two leaves.

        Returns violators and total.
        '''
        from abjad.tools import spannertools
        violators = []
        total = 0
        prototype = (spannertools.Hairpin,)
        hairpins = argument._get_descendants()._get_spanners(prototype)
        for hairpin in hairpins:
            if len(hairpin._get_leaves()) <= 1:
                violators.append(hairpin)
            total += 1
        return violators, total

    @staticmethod
    def check_tied_rests(argument=None):
        r'''Checks to make sure there are no tied rests.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import inspect_
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        for rest in iterate(argument).by_class(scoretools.Rest):
            if inspect_(rest).has_spanner(spannertools.Tie):
                violators.append(rest)
            total += 1
        return violators, total



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_append_spacer_skip_to_underfull_measure_01():
    r'''Handles measure prolation from time signature
    non-power-of-two denominator.
    '''

    measure = Measure((4, 12), "c'8 d'8 e'8 f'8")
    measure.implicit_scaling = True
    detach(TimeSignature, measure)
    time_signature = TimeSignature((5, 12))
    attach(time_signature, measure)
    assert measure.is_underfull

    scoretools.append_spacer_skip_to_underfull_measure(measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/12
            \scaleDurations #'(2 . 3) {
                c'8
                d'8
                e'8
                f'8
                s1 * 1/8
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_append_spacer_skip_to_underfull_measure_02():
    r'''Handles regular measure with no time signature prolation.
    '''

    measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
    detach(TimeSignature, measure)
    time_signature = TimeSignature((5, 8))
    attach(time_signature, measure)
    assert measure.is_underfull

    scoretools.append_spacer_skip_to_underfull_measure(measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/8
            c'8
            d'8
            e'8
            f'8
            s1 * 1/8
        }
        '''
        )

    assert inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import new
from abjad.tools.schemetools.Scheme import Scheme


class SchemePair(Scheme):
    r'''A Scheme pair.

    ::

        >>> schemetools.SchemePair('spacing', 4)
        SchemePair('spacing', 4)

    Initialize Scheme pairs with a tuple, two separate values
    or another Scheme pair.

    Scheme pairs are immutable.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        )

    ### INITIALIZER ##

    def __init__(self, *arguments, **keywords):
        if len(arguments) == 1 and isinstance(arguments[0], SchemePair):
            arguments = arguments[0]._value
        elif len(arguments) == 1 and isinstance(arguments[0], tuple):
            arguments = arguments[0][:]
        elif len(arguments) == 2:
            arguments = arguments
        elif len(arguments) == 0:
            arguments = (0, 1)
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arguments)
            raise TypeError(message)
        Scheme.__init__(self, *arguments, **keywords)

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats Scheme pair.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'lilypond'`.

        ::

            >>> scheme_pair = schemetools.SchemePair(-1, 1)

        ::

            >>> format(scheme_pair)
            "#'(-1 . 1)"

        ::

            >>> print(format(scheme_pair, 'storage'))
            schemetools.SchemePair(-1, 1)

        Returns string.
        '''
        superclass = super(SchemePair, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_value(self):
        from abjad.tools import schemetools
        assert len(self._value) == 2
        lhs = schemetools.Scheme.format_scheme_value(self._value[0])
        # need to force quotes around pairs like
        # \override #'(font-name . "Times")
        rhs = schemetools.Scheme.format_scheme_value(
            self._value[-1], force_quotes=True)
        return '({} . {})'.format(lhs, rhs)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        specification = super(SchemePair, self)._get_format_specification()
        return new(
            specification,
            repr_is_indented=False,
            storage_format_is_indented=False,
            )

    def _get_lilypond_format(self):
        return "#'%s" % self._formatted_value



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container_context_settings_01():
    r'''Noncontext containers bracket context settings at opening.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    set_(container).score.beam_exceptions = schemetools.SchemeVector()

    assert format(container) == stringtools.normalize(
        r'''
        {
            \set Score.beamExceptions = #'()
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Chord___init___01():
    r'''Initialize empty chord.
    '''

    chord = Chord([], (1, 4))
    assert format(chord) == "<>4"


def test_scoretools_Chord___init___02():
    r'''Initialize chord with pitch numbers.
    '''

    chord = Chord([2, 4, 5], (1, 4))
    assert format(chord) == "<d' e' f'>4"


def test_scoretools_Chord___init___03():
    r'''Initialize chord with pitch tokens.
    '''

    chord = Chord([('ds', 4), ('ef', 4)], (1, 4))
    assert format(chord) == "<ds' ef'>4"


def test_scoretools_Chord___init___04():
    r'''Initialize chord with pitches.
    '''

    pitches = []
    pitches.append(NamedPitch('ds', 4))
    pitches.append(NamedPitch('ef', 4))
    chord = Chord(pitches, (1, 4))
    assert format(chord) == "<ds' ef'>4"


def test_scoretools_Chord___init___05():
    r'''Initialize chord with pitches and pitch numbers together.
    '''

    pitches = [2, ('ef', 4), NamedPitch(4)]
    chord = Chord(pitches, (1, 4))
    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord___init___06():
    r'''Initialize chord with list of pitch names.
    '''

    pitches = ["d'", "ef'", "e'"]
    chord = Chord(pitches, (1, 4))
    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord___init___07():
    r'''Initialize chord with LilyPond input string.
    '''

    chord = Chord("<d' ef' e'>4")
    assert format(chord) == "<d' ef' e'>4"


def test_scoretools_Chord___init___08():
    r'''Initialize chord from skip.
    '''

    skip = scoretools.Skip('s8')
    chord = Chord(skip)

    assert format(skip) == 's8'
    assert format(chord) == '<>8'

    assert inspect_(skip).is_well_formed()
    assert inspect_(chord).is_well_formed()


def test_scoretools_Chord___init___09():
    r'''Initialize chord from tupletized skip.
    '''

    tuplet = Tuplet((2, 3), 's8 s8 s8')
    chord = Chord(tuplet[0])

    assert format(chord) == '<>8'
    assert inspect_(chord).get_parentage().parent is None
    assert inspect_(chord).is_well_formed()


def test_scoretools_Chord___init___10():
    r'''Initialize chord from containerized skip.
    '''

    tuplet = Voice('s8 s8 s8')
    chord = Chord(tuplet[0])

    assert format(chord) == '<>8'
    assert inspect_(chord).get_parentage().parent is None
    assert inspect_(chord).is_well_formed()



def test_scoretools_Chord___init___11():
    r'''Initialize chord from beamed skip.
    '''

    staff = Staff("c'8 [ s8 c'8 ]")
    chord = Chord(staff[1])

    assert format(chord) == '<>8'
    assert inspect_(chord).get_parentage().parent is None
    assert inspect_(chord).is_well_formed()


def test_scoretools_Chord___init___12():
    r'''Initialize chord from rest.
    '''

    rest = Rest('r8')
    chord = Chord(rest)

    assert format(rest) == 'r8'
    assert format(chord) == '<>8'
    assert inspect_(rest).is_well_formed()
    assert inspect_(chord).is_well_formed()


def test_scoretools_Chord___init___13():
    r'''Initialize chord from tupletized rest.
    '''

    tuplet = Tuplet((2, 3), 'r8 r8 r8')
    chord = Chord(tuplet[1])

    assert format(chord) == '<>8'
    assert inspect_(chord).is_well_formed()
    assert inspect_(chord).get_parentage().parent is None


def test_scoretools_Chord___init___14():
    r'''Initialize chord from note.
    '''

    note = Note("d'8")
    chord = Chord(note)

    assert format(note) == "d'8"
    assert format(chord) == "<d'>8"
    assert inspect_(note).is_well_formed()
    assert inspect_(chord).is_well_formed()


def test_scoretools_Chord___init___15():
    r'''Initialize chord from tupletized note.
    '''

    tuplet = Tuplet((2, 3), "c'8 c'8 c'8")
    chord = Chord(tuplet[1])

    assert format(chord) == "<c'>8"
    assert inspect_(chord).is_well_formed()
    assert inspect_(chord).get_parentage().parent is None


def test_scoretools_Chord___init___16():
    r'''Initialize chord from spanned note.
    '''

    staff = Staff("c'8 ( d'8 e'8 f'8 )")
    chord = Chord(staff[1])

    assert format(chord) == "<d'>8"
    assert inspect_(chord).is_well_formed()
    assert inspect_(chord).get_parentage().parent is None


def test_scoretools_Chord___init___17():
    r'''Initialize empty chord from LilyPond input string.
    '''

    chord = Chord('<>8.')

    assert format(chord) == '<>8.'
    assert not len(chord.note_heads)


def test_scoretools_Chord___init___18():
    r'''Initialize chord from LilyPond input string with forced and
    cautionary accidentals.
    '''

    chord = Chord('<c!? e? g! b>4')

    assert format(chord) == '<c!? e? g! b>4'


def test_scoretools_Chord___init___19():
    r'''Initialize chord from note with forced and cautionary accidentals.
    '''

    note = Note("c'!?4")
    chord = Chord(note)

    assert format(chord) == "<c'!?>4"


def test_scoretools_Chord___init___20():
    r'''Initialize chord from other chord.
    '''

    chord_1 = Chord("<c' e' g' bf'>4")
    chord_2 = Chord(chord_1, Duration(1, 8))

    assert format(chord_2) == "<c' e' g' bf'>8"


def test_scoretools_Chord___init___21():
    r'''Initialize chord with drum pitches.
    '''

    chord = Chord("<sn? bd! tamb>4")

    assert format(chord) == '<bassdrum! snare? tambourine>4'



# -*- coding: utf-8 -*-


def mutate(client):
    r'''Makes mutation agent.

    ..  container:: example

        Scales duration of last note notes in staff:

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                e'4
                d'4
                f'4
            }

        ::

            >>> mutate(staff[-2:]).scale(Multiplier(3, 2))
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                e'4
                d'4.
                f'4.
            }

    ..  container:: example

        Returns mutation agent:

        ::

            >>> mutate(staff[-2:])
            MutationAgent(client=Selection([Note("d'4."), Note("f'4.")]))

    '''
    from abjad.tools import agenttools
    return agenttools.MutationAgent(client)



# -*- coding: utf-8 -*-
import six


def is_string(argument):
    r'''Is true when `argument` is a string.

    Compatible under both Python 2.7.x and 3.x.
    '''
    return isinstance(argument, six.string_types)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_Mode___eq___01():

    mode_1 = tonalanalysistools.Mode('dorian')
    mode_2 = tonalanalysistools.Mode('dorian')
    mode_3 = tonalanalysistools.Mode('phrygian')

    assert      mode_1 == mode_1
    assert      mode_1 == mode_2
    assert not mode_1 == mode_3
    assert      mode_2 == mode_1
    assert      mode_2 == mode_2
    assert not mode_2 == mode_3
    assert not mode_3 == mode_1
    assert not mode_3 == mode_2
    assert      mode_3 == mode_3


def test_tonalanalysistools_Mode___eq___02():
    r'''Synonym modes do not compare equal, by definition.
    '''

    major = tonalanalysistools.Mode('major')
    ionian = tonalanalysistools.Mode('ionian')

    assert not major == ionian
    assert not ionian == major



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import inspect_
from abjad.tools.selectortools.Inequality import Inequality


class DurationInequality(Inequality):
    r'''Duration inequality.

    ::

        >>> inequality = selectortools.DurationInequality('<', (3, 4))
        >>> f(inequality)
        selectortools.DurationInequality(
            operator_string='<',
            duration=durationtools.Duration(3, 4),
            )

    ::

        >>> inequality(Duration(1, 2))
        True

    ::

        >>> inequality(Note("c'4"))
        True

    ::

        >>> inequality(Container("c'1 d'1"))
        False

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Inequalities'

    __slots__ = (
        '_duration',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        operator_string='<',
        duration=mathtools.Infinity(),
        ):
        Inequality.__init__(
            self,
            operator_string=operator_string,
            )
        infinities = (
            mathtools.Infinity(),
            mathtools.NegativeInfinity(),
            )
        if duration not in infinities:
            duration = durationtools.Duration(duration)
            assert 0 <= duration
        self._duration = duration

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        r'''Calls duration inequality on `argument`.

        Returns true or false.
        '''
        if isinstance(argument, scoretools.Component):
            duration = inspect_(argument).get_duration()
        elif isinstance(argument, selectiontools.Selection):
            duration = argument.get_duration()
        else:
            duration = durationtools.Duration(argument)
        result = self._operator_function(duration, self._duration)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def duration(self):
        r'''Gets duration.

        Returns duration.
        '''
        return self._duration



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeNode_improper_parentage_01():

    container = datastructuretools.TreeContainer()
    assert container.improper_parentage == (container,)

    leaf = datastructuretools.TreeNode()
    assert leaf.improper_parentage == (leaf,)

    container.append(leaf)
    assert leaf.improper_parentage == (leaf, container)

    subcontainer = datastructuretools.TreeContainer([leaf])
    container.append(subcontainer)
    assert leaf.improper_parentage == (leaf, subcontainer, container)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___neg___01():

    interval = pitchtools.NamedInterval('minor', 3)
    assert -interval == pitchtools.NamedInterval('minor', -3)


def test_pitchtools_NamedInterval___neg___02():

    interval = pitchtools.NamedInterval('minor', -3)
    assert -interval == pitchtools.NamedInterval('minor', 3)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitch___eq___01():

    pitch_1 = pitchtools.NumberedPitch(12)
    pitch_2 = pitchtools.NumberedPitch(12)
    pitch_3 = pitchtools.NumberedPitch(13)

    assert pitch_1 == pitch_1
    assert pitch_1 == pitch_2
    assert not pitch_1 == pitch_3

    assert pitch_2 == pitch_1
    assert pitch_2 == pitch_2
    assert not pitch_2 == pitch_3

    assert not pitch_3 == pitch_1
    assert not pitch_3 == pitch_2
    assert pitch_3 == pitch_3



# -*- coding: utf -*-
from abjad import *


def test_rhythmmakertools_Pattern_matches_index_01():

    mask = patterntools.Pattern(
        indices=[0, 1],
        period=None,
        )

    length = 9
    assert mask.matches_index(0, length)
    assert mask.matches_index(1, length)
    assert not mask.matches_index(2, length)
    assert not mask.matches_index(3, length)
    assert not mask.matches_index(4, length)
    assert not mask.matches_index(5, length)
    assert not mask.matches_index(6, length)
    assert not mask.matches_index(7, length)


def test_rhythmmakertools_Pattern_matches_index_02():

    mask = patterntools.Pattern(
        indices=[0, 1],
        period=4,
        )

    length = 9
    assert mask.matches_index(0, length)
    assert mask.matches_index(1, length)
    assert not mask.matches_index(2, length)
    assert not mask.matches_index(3, length)
    assert mask.matches_index(4, length)
    assert mask.matches_index(5, length)
    assert not mask.matches_index(6, length)
    assert not mask.matches_index(7, length)


def test_rhythmmakertools_Pattern_matches_index_03():

    mask = patterntools.Pattern(
        indices=[0, 1, 2, 3, 4, 5, 6, 7],
        period=None,
        )

    length = 9
    assert mask.matches_index(0, length)
    assert mask.matches_index(1, length)
    assert mask.matches_index(2, length)
    assert mask.matches_index(3, length)
    assert mask.matches_index(4, length)
    assert mask.matches_index(5, length)
    assert mask.matches_index(6, length)
    assert mask.matches_index(7, length)


def test_rhythmmakertools_Pattern_matches_index_04():

    mask = patterntools.Pattern(
        indices=[0, 1, 2, 3, 4, 5, 6, 7],
        period=None,
        )

    length = 2
    assert mask.matches_index(0, length)
    assert mask.matches_index(1, length)
    assert not mask.matches_index(2, length)
    assert not mask.matches_index(3, length)
    assert not mask.matches_index(4, length)
    assert not mask.matches_index(5, length)
    assert not mask.matches_index(6, length)
    assert not mask.matches_index(7, length)


def test_rhythmmakertools_Pattern_matches_index_05():

    mask = patterntools.Pattern(
        indices=[-2, -1],
        period=None,
        )

    length = 4
    assert not mask.matches_index(0, length)
    assert not mask.matches_index(1, length)
    assert mask.matches_index(2, length)
    assert mask.matches_index(3, length)
    assert not mask.matches_index(4, length)
    assert not mask.matches_index(5, length)
    assert not mask.matches_index(6, length)
    assert not mask.matches_index(7, length)


def test_rhythmmakertools_Pattern_matches_index_06():

    mask = patterntools.Pattern(
        indices=[-2, -1],
        period=4,
        )

    length = 4
    assert not mask.matches_index(0, length)
    assert not mask.matches_index(1, length)
    assert mask.matches_index(2, length)
    assert mask.matches_index(3, length)
    assert not mask.matches_index(4, length)
    assert not mask.matches_index(5, length)
    assert mask.matches_index(6, length)
    assert mask.matches_index(7, length)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment___mul___01():

    pitch_class_segment_1 = pitchtools.PitchClassSegment([0, 1, 11, 9])
    pitch_class_segment_2 = pitch_class_segment_1 * 2

    assert pitch_class_segment_2 == \
        pitchtools.PitchClassSegment([0, 1, 11, 9, 0, 1, 11, 9])



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import schemetools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import override


def make_solid_text_spanner_with_nib(
    left_text,
    direction=Up,
    ):
    r'''Makes solid text spanner with nib at right.

    ..  container:: example

        Solid text spanner forced above staff:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.make_solid_text_spanner_with_nib(
            ...     'foo',
            ...     direction=Up,
            ...     )
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \override TextSpanner.bound-details.left.text = \markup { foo }
                \override TextSpanner.bound-details.right-broken.text = ##f
                \override TextSpanner.bound-details.right.text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner.dash-fraction = #1
                \override TextSpanner.direction = #up
                c'8 \startTextSpan
                d'8
                e'8
                f'8 \stopTextSpan
                \revert TextSpanner.bound-details
                \revert TextSpanner.dash-fraction
                \revert TextSpanner.direction
            }

    ..  container:: example

        Solid text spanner forced below staff:

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.make_solid_text_spanner_with_nib(
            ...     'foo',
            ...     direction=Down,
            ...     )
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \override TextSpanner.bound-details.left.text = \markup { foo }
                \override TextSpanner.bound-details.right-broken.text = ##f
                \override TextSpanner.bound-details.right.text = \markup {
                    \draw-line
                        #'(0 . 1)
                    }
                \override TextSpanner.dash-fraction = #1
                \override TextSpanner.direction = #down
                c'8 \startTextSpan
                d'8
                e'8
                f'8 \stopTextSpan
                \revert TextSpanner.bound-details
                \revert TextSpanner.dash-fraction
                \revert TextSpanner.direction
            }

    Returns text spanner.
    '''
    from abjad.tools import spannertools
    assert direction in (Up, Down)

    text_spanner = spannertools.TextSpanner()
    left_text = markuptools.Markup(left_text)
    override(text_spanner).text_spanner.bound_details__left__text = left_text
    if direction == Up:
        pair = schemetools.SchemePair(0, -1)
    else:
        pair = schemetools.SchemePair(0, 1)
    right_text = markuptools.Markup(
        markuptools.MarkupCommand('draw-line', pair))
    override(text_spanner).text_spanner.bound_details__right__text = right_text
    override(text_spanner).text_spanner.bound_details__right_broken__text = \
        False
    override(text_spanner).text_spanner.dash_fraction = 1
    override(text_spanner).text_spanner.direction = direction

    return text_spanner



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Rest___init___01():
    r'''Initialize rest from LilyPond input string.
    '''

    rest = Rest('r8.')

    assert rest.written_duration == Duration(3, 16)


def test_scoretools_Rest___init___02():
    r'''Initialize rest from other rest.
    '''

    rest_1 = Rest('r4')
    attach(Multiplier(1, 2), rest_1)
    override(rest_1).staff.note_head.color = 'red'
    rest_2 = Rest(rest_1)

    assert isinstance(rest_1, Rest)
    assert isinstance(rest_2, Rest)
    assert format(rest_1) == format(rest_2)
    assert rest_1 is not rest_2


def test_scoretools_Rest___init___03():
    r'''Initialize rest from chord.
    '''

    chord = Chord([2, 3, 4], (1, 4))
    rest = Rest(chord)

    assert isinstance(rest, Rest)
    assert dir(chord) == dir(Chord([2, 3, 4], (1, 4)))
    assert dir(rest) == dir(Rest((1, 4)))
    assert rest.written_duration == chord.written_duration


def test_scoretools_Rest___init___04():
    r'''Initialize rest from tupletized chord.
    '''

    chord = Chord([2, 3, 4], Duration(1, 4))
    tuplet = Tuplet((2, 3), 3 * chord)
    rest = Rest(tuplet[0])

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        ''',
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___05():
    r'''Initialize rest from beamed chord.
    '''

    chord = Chord([2, 3, 4], Duration(1, 4))
    staff = Staff(3 * chord)
    beam = Beam()
    attach(beam, staff[:])
    rest = Rest(staff[0])

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        ''',
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___06():
    r'''Initialize rest from skip.
    '''

    skip = scoretools.Skip('s4')
    rest = Rest(skip)

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        ''',
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___07():
    r'''Initialize rest from tupletted skip.
    '''

    skip = scoretools.Skip('s4')
    tuplet = Tuplet((2, 3), 3 * skip)
    rest = Rest(tuplet[0])

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        ''',
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___08():
    r'''Initialize rest from beamed skip.
    '''

    skip = scoretools.Skip('s8')
    staff = Staff("c'8 [ s4 c'd ]")
    rest = Rest(staff[1])

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        '''
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___09():
    r'''Initialize rest from note.
    '''

    note = Note("c'4")
    rest = Rest(note)

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        '''
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___10():
    r'''Initialize rest from tupletized note.
    '''

    tuplet = Tuplet((2, 3), "c'4 d'4 e'4")
    rest = Rest(tuplet[0])

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        '''
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___11():
    r'''Initialize rest from beamed note.
    '''

    staff = Staff("c'8 [ d'8 e'8 ]")
    rest = Rest(staff[0])

    assert format(rest) == stringtools.normalize(
        r'''
        r8
        '''
        )

    assert inspect_(rest).is_well_formed()


def test_scoretools_Rest___init___12():
    r'''Initialize multiple rests from spanned notes.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    for note in voice:
        rest = Rest(note)
        mutate(note).replace(rest)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            r8 [
            r8
            r8
            r8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Rest___init___13():
    '''Initializes rest from empty input.
    '''

    rest = Rest()

    assert format(rest) == stringtools.normalize(
        r'''
        r4
        '''
        )

    assert inspect_(rest).is_well_formed()



# -*- coding: utf-8 -*-
import copy
from abjad.tools.datastructuretools.TypedList import TypedList


class PitchRangeList(TypedList):
    r"""Pitch range list.

    ..  container:: example

        Two pitch ranges:

        ::

            >>> ranges = pitchtools.PitchRangeList([
            ...     '[C3, C6]',
            ...     '[C4, C6]',
            ...     ])

        ::

            >>> f(ranges)
            pitchtools.PitchRangeList(
                [
                    pitchtools.PitchRange(
                        range_string='[C3, C6]',
                        ),
                    pitchtools.PitchRange(
                        range_string='[C4, C6]',
                        ),
                    ]
                )

        ::

            >>> show(ranges) # doctest: +SKIP

        ..  doctest::

            >>> lilypond_file = ranges.__illustrate__()
            >>> f(lilypond_file[Score])
            \new Score \with {
                \override BarLine.stencil = ##f
                \override Glissando.thickness = #2
                \override SpanBar.stencil = ##f
                \override TimeSignature.stencil = ##f
            } <<
                \new PianoStaff <<
                    \context Staff = "treble" {
                        \clef "treble"
                        s1 * 1/4
                        s1 * 1/4
                        c'1 * 1/4 \glissando
                        c'''1 * 1/4
                    }
                    \context Staff = "bass" {
                        \clef "bass"
                        c1 * 1/4 \glissando
                        \change Staff = treble
                        c'''1 * 1/4
                        s1 * 1/4
                        s1 * 1/4
                    }
                >>
            >>

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __illustrate__(self):
        r"""Illustrates pitch ranges.

        ..  container:: example

            ::

                >>> ranges = pitchtools.PitchRangeList([
                ...     '[C3, C6]',
                ...     '[C4, C6]',
                ...     ])

            ::

                >>> show(ranges) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = ranges.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score \with {
                    \override BarLine.stencil = ##f
                    \override Glissando.thickness = #2
                    \override SpanBar.stencil = ##f
                    \override TimeSignature.stencil = ##f
                } <<
                    \new PianoStaff <<
                        \context Staff = "treble" {
                            \clef "treble"
                            s1 * 1/4
                            s1 * 1/4
                            c'1 * 1/4 \glissando
                            c'''1 * 1/4
                        }
                        \context Staff = "bass" {
                            \clef "bass"
                            c1 * 1/4 \glissando
                            \change Staff = treble
                            c'''1 * 1/4
                            s1 * 1/4
                            s1 * 1/4
                        }
                    >>
                >>

        Returns LilyPond file.
        """
        import abjad
        start_note_clefs = []
        stop_note_clefs = []
        for pitch_range in self.items:
            start_note_clef = abjad.Clef.from_selection(
                pitch_range.start_pitch)
            start_note_clefs.append(start_note_clef)
            stop_note_clef = abjad.Clef.from_selection(
                pitch_range.stop_pitch)
            stop_note_clefs.append(stop_note_clef)
        if start_note_clefs == stop_note_clefs:
            clef = start_note_clefs[0]
            staff = abjad.Staff()
            abjad.attach(clef, staff)
            score = abjad.Score([staff])
            for pitch_range in self.items:
                start_note = abjad.Note(pitch_range.start_pitch, 1)
                stop_note = abjad.Note(pitch_range.stop_pitch, 1)
                notes = [start_note, stop_note]
                glissando = abjad.Glissando()
                staff.extend(notes)
                abjad.attach(glissando, notes)
        else:
            result = abjad.scoretools.make_empty_piano_score()
            score, treble_staff, bass_staff = result
            for pitch_range in self.items:
                start_note = abjad.Note(pitch_range.start_pitch, 1)
                start_note_clef = abjad.Clef.from_selection(
                    pitch_range.start_pitch)
                stop_note = abjad.Note(pitch_range.stop_pitch, 1)
                stop_note_clef = abjad.Clef.from_selection(
                    pitch_range.stop_pitch)
                notes = [start_note, stop_note]
                glissando = abjad.Glissando()
                skips = 2 * abjad.Skip(1)
                treble_clef = abjad.Clef('treble')
                bass_clef = abjad.Clef('bass')
                if start_note_clef == stop_note_clef == treble_clef:
                    treble_staff.extend(notes)
                    bass_staff.extend(skips)
                elif start_note_clef == stop_note_clef == bass_clef:
                    bass_staff.extend(notes)
                    treble_staff.extend(skips)
                else:
                    assert start_note_clef == bass_clef
                    assert stop_note_clef == treble_clef
                    bass_staff.extend(notes)
                    treble_staff.extend(skips)
                    staff_change = abjad.StaffChange(treble_staff)
                    abjad.attach(staff_change, stop_note)
                abjad.attach(glissando, notes)
        for leaf in abjad.iterate(score).by_leaf():
            multiplier = abjad.Multiplier(1, 4)
            abjad.attach(multiplier, leaf)
        abjad.override(score).bar_line.stencil = False
        abjad.override(score).span_bar.stencil = False
        abjad.override(score).glissando.thickness = 2
        abjad.override(score).time_signature.stencil = False
        lilypond_file = abjad.LilyPondFile.new(score)
        lilypond_file.items.remove(lilypond_file['layout'])
        lilypond_file.items.remove(lilypond_file['paper'])
        lilypond_file.header_block.tagline = False
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        def coerce_(argument):
            if isinstance(argument, str):
                range_ = pitchtools.PitchRange(argument)
            elif isinstance(argument, tuple):
                range_ = pitchtools.PitchRange.from_pitches(*argument)
            elif isinstance(argument, pitchtools.PitchRange):
                range_ = copy.copy(argument)
            else:
                raise TypeError(argument)
            return range_
        from abjad.tools import pitchtools
        return coerce_



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def apply_final_bar_lines(score):
    r'''Applies final bar lines to score.
    '''

    for voice in iterate(score).by_class(scoretools.Voice):
        bar_line = indicatortools.BarLine('|.')
        attach(bar_line, voice[-1])


# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_AbjadConfiguration_get_text_editor_01():
    assert isinstance(
        systemtools.AbjadConfiguration.get_text_editor(),
        str,
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadList___contains___01():

    chord = Chord("<ef' cs'' f''>4")

    assert 17 in chord.note_heads
    assert 17.0 in chord.note_heads
    assert NamedPitch(17) in chord.note_heads
    assert NamedPitch("f''") in chord.note_heads
    assert chord.note_heads[1] in chord.note_heads
    assert scoretools.NoteHead("f''") in chord.note_heads


def test_scoretools_NoteHeadList___contains___02():

    chord = Chord("<ef' cs'' f''>4")

    assert not 18 in chord.note_heads
    assert not 18.0 in chord.note_heads
    assert not NamedPitch(18) in chord.note_heads
    assert not NamedPitch("fs''") in chord.note_heads
    assert not scoretools.NoteHead(18) in chord.note_heads
    assert not scoretools.NoteHead("fs''") in chord.note_heads



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitchClass___add___01():

    npc = pitchtools.NamedPitchClass('c')

    new = npc + pitchtools.NamedInterval('perfect', 1)
    new == pitchtools.NamedPitchClass('c')

    new = npc + pitchtools.NamedInterval('minor', 2)
    new == pitchtools.NamedPitchClass('df')

    new = npc + pitchtools.NamedInterval('minor', -2)
    new == pitchtools.NamedPitchClass('b')

    new = npc + pitchtools.NamedInterval('major', 2)
    new == pitchtools.NamedPitchClass('d')

    new = npc + pitchtools.NamedInterval('major', -2)
    new == pitchtools.NamedPitchClass('bf')



# -*- coding: utf-8 -*-
from abjad import *


def test_topleveltools_override_01():

    note = Note("c'4")
    override(note).accidental.color = 'red'
    assert override(note).accidental.color == 'red'



# -*- coding: utf-8 -*-
from __future__ import print_function
import importlib
import inspect
import sys
import textwrap
import types
from abjad.tools import abctools
from abjad.tools import systemtools
from abjad.tools.topleveltools import new
from sphinx.util.console import bold, red


class CodeBlock(abctools.AbjadValueObject):
    r'''A code block.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Internals'

    ### CLASS VARIABLES ###

    __slots__ = (
        '_code_block_specifier',
        '_console',
        '_current_lines',
        '_document_source',
        '_executed_lines',
        '_image_layout_specifier',
        '_image_render_specifier',
        '_options',
        '_output_proxies',
        '_source_lines',
        '_starting_line_number',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        input_file_contents,
        code_block_specifier=None,
        document_source=None,
        executed_lines=None,
        image_layout_specifier=None,
        image_render_specifier=None,
        starting_line_number=None,
        ):
        self._code_block_specifier = code_block_specifier
        self._current_lines = []
        self._document_source = document_source
        if executed_lines is not None:
            executed_lines = tuple(executed_lines)
        self._executed_lines = executed_lines
        self._image_layout_specifier = image_layout_specifier
        self._image_render_specifier = image_render_specifier
        self._output_proxies = []
        self._source_lines = tuple(input_file_contents)
        self._starting_line_number = starting_line_number

    ### PUBLIC METHODS ###

    def as_docutils(
        self,
        configuration=None,
        output_directory=None,
        ):
        output_proxies = self.filter_output_proxies()
        result = []
        if output_proxies:
            for output_proxy in output_proxies:
                subresult = output_proxy.as_docutils(
                    configuration=configuration,
                    output_directory=output_directory,
                    )
                result.extend(subresult)
        return result

    def as_latex(
        self,
        configuration=None,
        output_directory=None,
        relative_output_directory=None,
        ):
        from abjad.tools import abjadbooktools
        result = []
        configuration = configuration or {}
        latex_configuration = configuration.get('latex', {})
        output_start_delimiter = latex_configuration.get(
            'output-start-delimiter',
            ('%%% ABJADBOOK START %%%',),
            )
        output_stop_delimiter = latex_configuration.get(
            'output-stop-delimiter',
            ('%%% ABJADBOOK END %%%',),
            )
        output_proxies = self.filter_output_proxies()
        if output_proxies:
            result.extend(output_start_delimiter)
            before = latex_configuration.get('before-code-block', ())
            result.extend(before)
            for i, output_proxy in enumerate(output_proxies):
                lines = output_proxy.as_latex(
                    configuration=configuration,
                    output_directory=output_directory,
                    relative_output_directory=relative_output_directory,
                    )
                string = '\n'.join(lines)
                if isinstance(output_proxy, abjadbooktools.ImageOutputProxy):
                    if i < len(output_proxies) - 1:
                        next_proxy = output_proxies[i + 1]
                        if isinstance(next_proxy, abjadbooktools.ImageOutputProxy):
                            string += r'\\'
                result.append(string)
            after = latex_configuration.get('after-code-block', ())
            result.extend(after)
            result.extend(output_stop_delimiter)
            result = '\n'.join(result)
            result = [result]
        return result

    def filter_output_proxies(self):
        from abjad.tools import abjadbooktools
        code_block_specifier = self.code_block_specifier
        if code_block_specifier is None:
            code_block_specifier = abjadbooktools.CodeBlockSpecifier()
        output_proxies = list(self.output_proxies)
        if code_block_specifier.hide:
            output_proxies = [
                _ for _ in output_proxies
                if not isinstance(_, abjadbooktools.CodeOutputProxy)
                ]
        return output_proxies

    def flush(self):
        pass

    @staticmethod
    def from_docutils_abjad_import_block(block):
        from abjad.tools import abjadbooktools
        code_address = block['path']
        module_name, sep, attr_name = code_address.rpartition(':')
        module = importlib.import_module(module_name)
        attr = getattr(module, attr_name)
        input_file_contents = inspect.getsource(attr)
        if sys.version_info[0] == 2:
            input_file_contents = input_file_contents.decode('utf-8')
        input_file_contents = input_file_contents.splitlines()
        executed_lines = 'from {} import {}'.format(
            module_name,
            attr_name,
            )
        executed_lines = (executed_lines,)
        cleaned_options = {}
        for key, value in block.attlist():
            key = key.replace('-', '_')
            cleaned_options[key] = value
        code_block_specifier = abjadbooktools.CodeBlockSpecifier.from_options(
            **cleaned_options)
        image_layout_specifier = abjadbooktools.ImageLayoutSpecifier.from_options(
            **cleaned_options)
        image_render_specifier = abjadbooktools.ImageRenderSpecifier.from_options(
            **cleaned_options)
        code_block = abjadbooktools.CodeBlock(
            code_block_specifier=code_block_specifier,
            executed_lines=executed_lines,
            image_layout_specifier=image_layout_specifier,
            image_render_specifier=image_render_specifier,
            input_file_contents=input_file_contents,
            starting_line_number=block.line,
            )
        return code_block

    @staticmethod
    def from_docutils_abjad_input_block(block):
        from abjad.tools import abjadbooktools
        literal_block = block[0]
        text_node = literal_block[0]
        input_file_contents = tuple(text_node.splitlines())
        cleaned_options = {}
        for key, value in block.attlist():
            key = key.replace('-', '_')
            cleaned_options[key] = value
        code_block_specifier = abjadbooktools.CodeBlockSpecifier.from_options(
            **cleaned_options)
        image_layout_specifier = abjadbooktools.ImageLayoutSpecifier.from_options(
            **cleaned_options)
        image_render_specifier = abjadbooktools.ImageRenderSpecifier.from_options(
            **cleaned_options)
        code_block = abjadbooktools.CodeBlock(
            code_block_specifier=code_block_specifier,
            image_layout_specifier=image_layout_specifier,
            image_render_specifier=image_render_specifier,
            input_file_contents=input_file_contents,
            starting_line_number=literal_block.line,
            )
        return code_block

    @staticmethod
    def from_docutils_literal_block(block):
        from abjad.tools import abjadbooktools
        input_file_contents = []
        for line in block[0].splitlines():
            if line.startswith(('>>>', '...')):
                input_file_contents.append(line[4:])
        document_source = None
        parent = block.parent
        while not document_source and getattr(parent, 'parent'):
            document_source = getattr(parent, 'source', None)
            parent = parent.parent
        code_block_specifier = abjadbooktools.CodeBlockSpecifier(
            allow_exceptions=True,
            )
        code_block = abjadbooktools.CodeBlock(
            code_block_specifier=code_block_specifier,
            document_source=document_source,
            input_file_contents=input_file_contents,
            starting_line_number=block.line,
            )
        return code_block

    @staticmethod
    def from_latex_abjad_block(
        input_file_contents,
        starting_line_number=None,
        **options
        ):
        from abjad.tools import abjadbooktools
        cleaned_options = {}
        for key, value in options.items():
            key = key.replace('-', '_')
            cleaned_options[key] = value
        code_block_specifier = abjadbooktools.CodeBlockSpecifier.from_options(
            **cleaned_options)
        image_layout_specifier = abjadbooktools.ImageLayoutSpecifier.from_options(
            **cleaned_options)
        image_render_specifier = abjadbooktools.ImageRenderSpecifier.from_options(
            **cleaned_options)
        code_block = abjadbooktools.CodeBlock(
            code_block_specifier=code_block_specifier,
            image_layout_specifier=image_layout_specifier,
            image_render_specifier=image_render_specifier,
            input_file_contents=input_file_contents,
            starting_line_number=starting_line_number,
            )
        return code_block

    @staticmethod
    def from_latex_abjadextract_block(
        source_line,
        starting_line_number=None,
        **options
        ):
        from abjad.tools import abjadbooktools
        code_address = source_line.partition('<abjadextract ')[-1].split()[0]
        module_name, sep, attr_name = code_address.rpartition(':')
        module = importlib.import_module(module_name)
        attr = getattr(module, attr_name)
        input_file_contents = inspect.getsource(attr).splitlines()
        executed_lines = 'from {} import {}'.format(
            module_name,
            attr_name,
            )
        executed_lines = (executed_lines,)
        cleaned_options = {}
        for key, value in options.items():
            key = key.replace('-', '_')
            cleaned_options[key] = value
        code_block_specifier = abjadbooktools.CodeBlockSpecifier.from_options(
            **cleaned_options)
        image_layout_specifier = abjadbooktools.ImageLayoutSpecifier.from_options(
            **cleaned_options)
        image_render_specifier = abjadbooktools.ImageRenderSpecifier.from_options(
            **cleaned_options)
        code_block = abjadbooktools.CodeBlock(
            code_block_specifier=code_block_specifier,
            executed_lines=executed_lines,
            image_layout_specifier=image_layout_specifier,
            image_render_specifier=image_render_specifier,
            input_file_contents=input_file_contents,
            starting_line_number=starting_line_number,
            )
        return code_block

    def interpret(self, console):
        from abjad.tools import abjadbooktools
        code_block_specifier = self.code_block_specifier
        if code_block_specifier is None:
            code_block_specifier = abjadbooktools.CodeBlockSpecifier()
        self._console = console
        is_incomplete_statement = False
        input_file_contents = self.input_file_contents
        self.output_proxies[:] = []
        self.current_lines[:] = []
        self.setup_capture_hooks(console)
        if self.executed_lines:
            for executed_line in self.executed_lines:
                is_incomplete_statement = console.push(executed_line)
            code_output_proxy = abjadbooktools.CodeOutputProxy(
                input_file_contents,
                code_block_specifier=self.code_block_specifier,
                )
            self.output_proxies.insert(0, code_output_proxy)
        else:
            result = '>>> '
            for line_number, line in enumerate(input_file_contents, 1):
                result += line
                self.current_lines.append(result)
                if code_block_specifier.strip_prompt and line.strip() == '':
                    continue
                else:
                    is_incomplete_statement = self.push_line_to_console(
                        line, console, line_number)
                if not is_incomplete_statement:
                    result = '>>> '
                    self.setup_capture_hooks(console)
                else:
                    result = '... '
        if is_incomplete_statement:
            self.current_lines.append(result)
            is_incomplete_statement = self.push_line_to_console(
                '\n', console, line_number)
        self.push_code_output_proxy()
        if code_block_specifier.hide:
            self.output_proxies[:] = [
                _ for _ in self.output_proxies
                if not isinstance(_, abjadbooktools.CodeOutputProxy)
                ]
        self._console = None

    def push_asset_output_proxy(self, asset_output_proxy):
        self.push_code_output_proxy()
        self.output_proxies.append(asset_output_proxy)

    def push_code_output_proxy(self):
        from abjad.tools import abjadbooktools
        strip_prompt = None
        if self.code_block_specifier is not None:
            strip_prompt = self.code_block_specifier.strip_prompt
        if not self.current_lines:
            return
        if strip_prompt:
            self.current_lines[:] = [
                _[4:].rstrip() for _ in self.current_lines
                if _.startswith(('>>> ', '... '))
                ]
            while not self.current_lines[-1]:
                self.current_lines.pop()
        code_output_proxy = abjadbooktools.CodeOutputProxy(
            self.current_lines,
            code_block_specifier=self.code_block_specifier,
            )
        self.output_proxies.append(code_output_proxy)
        self.current_lines[:] = []

    def push_line_to_console(self, line, console, line_number):
        from abjad.tools import abjadbooktools
        allow_exceptions = None
        if self.code_block_specifier is not None:
            allow_exceptions = self.code_block_specifier.allow_exceptions
        with systemtools.RedirectedStreams(self, self):
            is_incomplete_statement = console.push(line)
        if console.errored:
            if allow_exceptions:
                console.unregister_error()
            else:
                message = 'Abjad-book error on '
                if self.document_source:
                    message += str(self.document_source)
                    message += ':{}'
                else:
                    message += 'line number {}'
                line_number += self.starting_line_number
                message = message.format(line_number)
                message = bold(red(message))
                message += '\n    '
                message = message + '\n    '.join(self.current_lines)
                raise abjadbooktools.AbjadBookError(message)
        return is_incomplete_statement

    def setup_capture_hooks(self, console):
        prototype = (types.MethodType, types.FunctionType)
        if isinstance(console.locals['graph'], prototype):
            console.locals['graph'] = self.graph
        console.locals['play'] = self.play
        console.locals['__builtins__']['print'] = self.print
        console.locals['__builtins__']['quit'] = self.quit
        console.locals['show'] = self.show
        topleveltools = console.locals['topleveltools']
        topleveltools.__dict__['graph'] = self.graph
        topleveltools.__dict__['play'] = self.play
        topleveltools.__dict__['show'] = self.show

    def write(self, string):
        text_width = None
        if self.code_block_specifier is not None:
            text_width = self.code_block_specifier.text_width
        if not string:
            return
        if sys.version_info[0] == 2:
            string = string.decode('utf-8')
        if string.endswith('\n'):
            string = string[:-1]
        lines = string.splitlines()
        if text_width is None:
            self.current_lines.extend(lines)
        else:
            for line in lines:
                wrapped_lines = textwrap.wrap(line, text_width)
                self.current_lines.extend(wrapped_lines)

    ### PROXIES ###

    def graph(
        self,
        argument,
        layout='dot',
        graph_attributes=None,
        node_attributes=None,
        edge_attributes=None,
        **keywords
        ):
        r'''Proxies Abjad's toplevel `graph()` function.
        '''
        from abjad.tools import abjadbooktools
        graph = argument.__graph__(**keywords)
        if graph_attributes:
            graph.attributes.update(graph_attributes)
        if node_attributes:
            graph.node_attributes.update(node_attributes)
        if edge_attributes:
            graph.edge_attributes.update(edge_attributes)
        output_proxy = abjadbooktools.GraphvizOutputProxy(
            graph,
            layout=layout,
            image_layout_specifier=self.image_layout_specifier,
            image_render_specifier=self.image_render_specifier,
            )
        self.push_asset_output_proxy(output_proxy)

    def play(self, argument):
        r'''Proxies Abjad's toplevel `play()` function.
        '''
        pass

    def print(self, *arguments, **keywords):
        r'''Proxies Python's builtin `print()` function.

        Proxying `print()` is necessary because the original print function
        makes multiple calls to `sys.stdout.write()` rather than one. In the
        context of abjad-book code block interpretation, each write operation
        adds a new line, hence the necessity of joining the `print()` arguments
        into a single string and then writing that.

        Note that this will produce unexpected results if attempting to do
        fancy operations like overwriting the current line.
        '''
        self.write(' '.join(str(_) for _ in arguments))

    def quit(self):
        r'''Proxies Python's builtin `quit()` function.
        '''
        pass

    def show(self, argument, return_timing=False, **keywords):
        r'''Proxies Abjad's toplevel `show()` function.
        '''
        from abjad.tools import abjadbooktools
        illustration = argument.__illustrate__(**keywords)
        default_stylesheet = None
        if (
            self._console is not None and
            self._console.document_handler is not None
            ):
            handler = self._console.document_handler
            default_stylesheet = handler.get_default_stylesheet()
        image_render_specifier = self.image_render_specifier
        if image_render_specifier is None:
            image_render_specifier = abjadbooktools.ImageRenderSpecifier()
        if image_render_specifier.stylesheet is None:
            image_render_specifier = new(
                image_render_specifier,
                stylesheet=default_stylesheet,
                )
        output_proxy = abjadbooktools.LilyPondOutputProxy(
            illustration,
            image_layout_specifier=self.image_layout_specifier,
            image_render_specifier=image_render_specifier,
            )
        self.push_asset_output_proxy(output_proxy)

    ### PUBLIC PROPERTIES ###

    @property
    def code_block_specifier(self):
        return self._code_block_specifier

    @property
    def current_lines(self):
        return self._current_lines

    @property
    def document_source(self):
        return self._document_source

    @property
    def executed_lines(self):
        return self._executed_lines

    @property
    def image_layout_specifier(self):
        return self._image_layout_specifier

    @property
    def image_render_specifier(self):
        return self._image_render_specifier

    @property
    def input_file_contents(self):
        return self._source_lines

    @property
    def output_proxies(self):
        return self._output_proxies

    @property
    def starting_line_number(self):
        return self._starting_line_number



# -*- coding: utf-8 -*-
from abjad import Fraction


def is_nonnegative_integer_power_of_two(argument):
    r'''Is true when `argument` is a nonnegative integer power of 2.
    Otherwise false.

    ..  container:: example

        ::

            >>> for n in range(10):
            ...     print(n, mathtools.is_nonnegative_integer_power_of_two(n))
            ... 
            0 True
            1 True
            2 True
            3 False
            4 True
            5 False
            6 False
            7 False
            8 True
            9 False

    Returns true or false.
    '''
    if isinstance(argument, int):
        return not bool(argument & (argument - 1))
    elif isinstance(argument, Fraction):
        return is_nonnegative_integer_power_of_two(
            argument.numerator * argument.denominator
            )
    else:
        return False



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_FixedDurationTuplet_preferred_denominator_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(4, 8), [])
    tuplet.extend("c'8 d'8 e'8 f'8 g'8 a'8")
    tuplet.preferred_denominator = 4

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 4/6 {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Note___init___01():
    r'''Initializes note from empty input.
    '''

    note = Note()

    assert format(note) == "c'4"


def test_scoretools_Note___init___02():
    r'''Initializes note with pitch in octave zero.
    '''

    note = Note(-37, (1, 4))

    assert format(note) == 'b,,,4'


def test_scoretools_Note___init___03():
    r'''Initializes note with non-assignable duration.
    '''

    pytest.raises(AssignabilityError, 'Note(0, (5, 8))')


def test_scoretools_Note___init___04():
    r'''Initializes note with LilyPond-style pitch string.
    '''

    note = Note('c,,', (1, 4))

    assert format(note) == 'c,,4'


def test_scoretools_Note___init___05():
    r'''Initializes note with complete LilyPond-style note string.
    '''

    note = Note('cs8.')

    assert format(note) == 'cs8.'


def test_scoretools_Note___init___06():
    r'''Initializes note from chord.
    '''

    chord = Chord([2, 3, 4], (1, 4))
    note = Note(chord)

    assert format(note) == stringtools.normalize(
        r'''
        d'4
        '''
        )

    assert inspect_(note).is_well_formed()


def test_scoretools_Note___init___07():
    r'''Initializes note from tupletized chord.
    '''

    chord = Chord([2, 3, 4], (1, 4))
    tuplet = Tuplet((2, 3), 3 * chord)
    note = Note(tuplet[0])

    assert format(note) == stringtools.normalize(
        r'''
        d'4
        '''
        )

    assert inspect_(note).is_well_formed()


def test_scoretools_Note___init___08():
    r'''Initializes note from beamed chord.
    '''

    chord = Chord([2, 3, 4], (1, 4))
    staff = Staff(3 * chord)
    beam = Beam()
    attach(beam, staff[:])
    note = Note(staff[0])

    assert format(note) == stringtools.normalize(
        r'''
        d'4
        '''
        )

    assert inspect_(note).is_well_formed()


def test_scoretools_Note___init___09():
    r'''Initializes note from rest.
    '''

    rest = Rest('r8')
    note = Note(rest)

    assert format(note) == stringtools.normalize(
        r'''
        8
        '''
        )

    assert inspect_(note).is_well_formed()


def test_scoretools_Note___init___10():
    r'''Initializes note from tupletized rest.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), Rest((1, 8)) * 3)
    d = tuplet[0].written_duration
    note = Note(tuplet[0])

    assert isinstance(tuplet[0], Rest)
    assert isinstance(note, Note)
    assert tuplet[0]._parent is tuplet
    assert tuplet[0].written_duration == d
    assert note._parent is None


def test_scoretools_Note___init___11():
    r'''Initializes note from beamed rest.
    '''

    staff = Staff([Note(0, (1, 8)), Rest((1, 8)), Note(0, (1, 8))])
    beam = Beam()
    attach(beam, staff[:])
    note = Note(staff[1])

    assert isinstance(staff[1], Rest)
    assert isinstance(note, Note)
    assert staff[1]._parent is staff
    assert note._parent is None


def test_scoretools_Note___init___12():
    r'''Initializes notes from skip.
    '''

    skip = scoretools.Skip((1, 8))
    d = skip.written_duration
    note = Note(skip)

    assert isinstance(note, Note)
    assert dir(skip) == dir(scoretools.Skip((1, 4)))
    assert dir(note) == dir(Note("c'4"))
    assert note._parent is None
    assert note.written_duration == d


def test_scoretools_Note___init___13():
    r'''Initializes note from tupletized skip.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), scoretools.Skip((1, 8)) * 3)
    d = tuplet[0].written_duration
    note = Note(tuplet[0])

    assert isinstance(tuplet[0], scoretools.Skip)
    assert isinstance(note, Note)
    assert tuplet[0]._parent is tuplet
    assert tuplet[0].written_duration == d
    assert note._parent is None


def test_scoretools_Note___init___14():
    r'''Initializes note from beamed skip.
    '''

    staff = Staff([Note(0, (1, 8)), scoretools.Skip((1, 8)), Note(0, (1, 8))])
    beam = Beam()
    attach(beam, staff[:])
    note = Note(staff[1])

    assert isinstance(staff[1], scoretools.Skip)
    assert isinstance(note, Note)
    assert staff[1]._parent is staff
    assert note._parent is None


def test_scoretools_Note___init___15():
    r'''Initializes note with cautionary accidental.
    '''

    note = Note("c'?4")

    assert format(note) == "c'?4"


def test_scoretools_Note___init___16():
    r'''Initializes note with forced accidental.
    '''

    note = Note("c'!4")

    assert format(note) == "c'!4"


def test_scoretools_Note___init___17():
    r'''Initializes note with both forced and cautionary accidental.
    '''

    note = Note("c'!?4")

    assert format(note) == "c'!?4"


def test_scoretools_Note___init___18():
    r'''Initializes note from chord with forced and cautionary accidental.
    '''

    chord = Chord("<c'!? e' g'>4")
    note = Note(chord)

    assert format(note) == "c'!?4"


def test_scoretools_Note___init___19():
    r'''Initialize note with drum pitch.
    '''

    note = Note('sn4')

    assert format(note) == 'snare4'



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_tonalanalysistools_RootedChordClass_cardinality_to_extent_01():

    assert tonalanalysistools.RootedChordClass.cardinality_to_extent(3) == 5
    assert tonalanalysistools.RootedChordClass.cardinality_to_extent(4) == 7
    assert tonalanalysistools.RootedChordClass.cardinality_to_extent(5) == 9
    assert tonalanalysistools.RootedChordClass.cardinality_to_extent(6) == 11
    assert tonalanalysistools.RootedChordClass.cardinality_to_extent(7) == 13


def test_tonalanalysistools_RootedChordClass_cardinality_to_extent_02():

    string = 'tonalanalysistools.RootedChordClass.cardinality_to_extent(1)'
    assert pytest.raises(Exception, string)

    string = 'tonalanalysistools.RootedChordClass.cardinality_to_extent(10)'
    assert pytest.raises(Exception, string)



# -*- coding: utf-8 -*-


def all_are_integer_equivalent(argument):
    '''Is true when `argument` is an iterable collection with
    integer-equivalent items. Otherwise false.

    ..  container:: example

        ::

            >>> items = [1, '2', 3.0, Fraction(4, 1)]
            >>> mathtools.all_are_integer_equivalent(items)
            True

        ::

            >>> mathtools.all_are_integer_equivalent([1, '2', 3.5, 4])
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return all(
            mathtools.is_integer_equivalent(_)
            for _ in argument
            )
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad import *
import copy


def test_scoretools_MultimeasureRest___copy___01():

    multi_measure_rest_1 = scoretools.MultimeasureRest((1, 4))
    multi_measure_rest_2 = copy.copy(multi_measure_rest_1)

    assert isinstance(multi_measure_rest_1, scoretools.MultimeasureRest)
    assert isinstance(multi_measure_rest_2, scoretools.MultimeasureRest)
    assert format(multi_measure_rest_1) == format(multi_measure_rest_2)
    assert multi_measure_rest_1 is not multi_measure_rest_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___init___01():
    r'''Initialize named interval from abbreviation.
    '''

    mdi = pitchtools.NamedInterval('+M3')
    assert mdi.quality_string == 'major'
    assert mdi.number == 3


def test_pitchtools_NamedInterval___init___02():
    r'''Can init from quality string and interval number.
    '''

    mdi = pitchtools.NamedInterval('major', 3)
    assert mdi.quality_string == 'major'
    assert mdi.number == 3


def test_pitchtools_NamedInterval___init___03():
    r'''Can init from other named interval instance.
    '''

    mdi = pitchtools.NamedInterval('major', 3)
    new = pitchtools.NamedInterval(mdi)

    assert mdi.quality_string == 'major'
    assert mdi.number == 3

    assert new.quality_string == 'major'
    assert new.number == 3

    assert new is not mdi
    assert new == mdi



# -*- coding: utf-8 -*-
from abjad import *


def test_timespantools_Timespan_stops_before_offset_01():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(-5)
    assert not timespan.stops_before_offset(offset)

def test_timespantools_Timespan_stops_before_offset_02():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(0)
    assert not timespan.stops_before_offset(offset)

def test_timespantools_Timespan_stops_before_offset_03():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(5)
    assert not timespan.stops_before_offset(offset)

def test_timespantools_Timespan_stops_before_offset_04():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(10)
    assert not timespan.stops_before_offset(offset)

def test_timespantools_Timespan_stops_before_offset_05():
    timespan = timespantools.Timespan(0, 10)
    offset = durationtools.Offset(15)
    assert timespan.stops_before_offset(offset)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__indicators__TimeSignature_01():

    target = Score([Staff([Note(0, 1)])])
    time_signature = TimeSignature((8, 8))
    attach(time_signature, target[0])

    assert format(target) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \time 8/8
                c'1
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result
    leaves = select(result).by_leaf()
    leaf = leaves[0]
    time_signatures = inspect_(leaf).get_indicators(TimeSignature)
    assert len(time_signatures) == 1



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_transpose_01():

    pitch_class_segment_1 = pitchtools.PitchClassSegment([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),
        pitchtools.NamedPitchClass('f'),
        pitchtools.NamedPitchClass('g'),])

    pitch_class_segment_2 = pitchtools.PitchClassSegment([
        pitchtools.NamedPitchClass('df'),
        pitchtools.NamedPitchClass('ef'),
        pitchtools.NamedPitchClass('f'),
        pitchtools.NamedPitchClass('gf'),
        pitchtools.NamedPitchClass('af'),])

    minor_second_ascending = pitchtools.NamedInterval('minor', 2)
    assert pitch_class_segment_1.transpose(minor_second_ascending) == \
        pitch_class_segment_2

    major_seventh_descending = pitchtools.NamedInterval('major', -7)
    assert pitch_class_segment_1.transpose(major_seventh_descending) == \
        pitch_class_segment_2

    minor_second_descending = pitchtools.NamedInterval('minor', -2)
    assert pitch_class_segment_2.transpose(minor_second_descending) == \
        pitch_class_segment_1

    major_seventh_ascending = pitchtools.NamedInterval('major', 7)
    assert pitch_class_segment_2.transpose(major_seventh_ascending) == \
        pitch_class_segment_1


def test_pitchtools_PitchClassSegment_transpose_02():

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.transpose(0) == \
        pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.transpose(1) == \
        pitchtools.PitchClassSegment([1, 7, 11, 5, 10, 3])
    assert pitch_class_segment.transpose(2) == \
        pitchtools.PitchClassSegment([2, 8, 0, 6, 11, 4])
    assert pitch_class_segment.transpose(3) == \
        pitchtools.PitchClassSegment([3, 9, 1, 7, 0, 5])
    assert pitch_class_segment.transpose(4) == \
        pitchtools.PitchClassSegment([4, 10, 2, 8, 1, 6])
    assert pitch_class_segment.transpose(5) == \
        pitchtools.PitchClassSegment([5, 11, 3, 9, 2, 7])



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_PitchedQEvent___init___01():

    q_event = quantizationtools.PitchedQEvent(130, [0, 1, 4])

    assert q_event.offset == durationtools.Offset(130)
    assert q_event.pitches == (
        NamedPitch(0),
        NamedPitch(1),
        NamedPitch(4)
        )
    assert q_event.attachments == ()


def test_quantizationtools_PitchedQEvent___init___02():

    q_event = quantizationtools.PitchedQEvent(
        durationtools.Offset(133, 5),
        [NamedPitch('fss')],
        attachments = ['foo', 'bar', 'baz']
        )

    assert q_event.offset == durationtools.Offset(133, 5)
    assert q_event.pitches == (NamedPitch('fss'),)
    assert q_event.attachments == ('foo', 'bar', 'baz')



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_get_spanner_01():

    container = Container("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, container[:-1])
    slur = Slur()
    attach(slur, container[:-1])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 [ (
            d'8
            e'8 ] )
            f'8
        }
        '''
        )

    string = 'inspect_(container[0]).get_spanner()'
    assert pytest.raises(Exception, string)

    assert inspect_(container[-1]).get_spanner() is None


def test_agenttools_InspectionAgent_get_spanner_02():

    staff = Staff(r"c'4 \times 2/3 { d'8 e'8 f'8 } g'2")
    leaves = select(staff).by_leaf()
    slur = Slur()
    attach(slur, leaves)
    for leaf in leaves:
        assert slur == inspect_(leaf).get_spanner(Slur, in_parentage=True)
